@@ -14,7 +14,7 @@ Including another URLconf
     2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
 """
 
-from bokeh.server.django import autoload
+from bokeh.server.django import autoload, static_extensions
 from django.apps import apps
 from django.contrib import admin
 from django.urls import path, include
@@ -33,4 +33,5 @@ bokeh_apps = [
     autoload("sliders", sliders_app.app),
 ]
 
+urlpatterns += static_extensions()
 urlpatterns += staticfiles_urlpatterns()
