@@ -6,14 +6,24 @@ ways.
 import ast
 import base64
 import logging # isort:skip
+import os
 
 from glob import glob
+from types import ModuleType
 
 from bokeh.command.subcommands.serve import Serve as _BkServe
 from bokeh.command.util import build_single_handler_applications
 
+from bokeh.application import Application
+from bokeh.application.handlers.document_lifecycle import DocumentLifecycleHandler
+from bokeh.application.handlers.function import FunctionHandler
+from bokeh.server.contexts import ApplicationContext
+from tornado.ioloop import PeriodicCallback, IOLoop
+from tornado.web import StaticFileHandler
+
 from ..auth import OAuthProvider
 from ..config import config
+from ..util import fullpath
 from ..io.rest import REST_PROVIDERS
 from ..io.reload import record_modules, watch
 from ..io.server import INDEX_HTML, get_static_routes, set_curdoc
@@ -57,6 +67,28 @@ def parse_vars(items):
     return dict((parse_var(item) for item in items))
 
 
+class AdminApplicationContext(ApplicationContext):
+
+    def __init__(self, application, unused_timeout=15000, **kwargs):
+        super().__init__(application, io_loop=IOLoop.current(), **kwargs)
+        self._unused_timeout = unused_timeout
+        self._cleanup_cb = None
+        self._loop.add_callback(self.run_load_hook)
+
+    async def cleanup_sessions(self):
+        await self._cleanup_sessions(self._unused_timeout)
+
+    def run_load_hook(self):
+        self._cleanup_cb = PeriodicCallback(self.cleanup_sessions, self._unused_timeout)
+        self._cleanup_cb.start()
+        super().run_load_hook()
+
+    def run_unload_hook(self):
+        if self._cleanup_cb:
+            self._cleanup_cb.stop()
+        super().run_unload_hook()
+
+
 class Serve(_BkServe):
 
     args = _BkServe.args + (
@@ -101,6 +133,11 @@ class Serve(_BkServe):
             type    = str,
             help    = "A random string used to encode the user information."
         )),
+        ('--oauth-error-template', dict(
+            action = 'store',
+            type    = str,
+            help    = "A random string used to encode the user information."
+        )),
         ('--oauth-expiry-days', dict(
             action  = 'store',
             type    = float,
@@ -132,12 +169,46 @@ class Serve(_BkServe):
             action  = 'store_true',
             help    = "Whether to execute scripts on startup to warm up the server."
         )),
+        ('--admin', dict(
+            action  = 'store_true',
+            help    = "Whether to add an admin panel."
+        )),
+        ('--profiler', dict(
+            action  = 'store',
+            type    = str,
+            help    = "The profiler to use by default, e.g. pyinstrument or snakeviz."
+        )),
         ('--autoreload', dict(
             action  = 'store_true',
             help    = "Whether to autoreload source when script changes."
+        )),
+        ('--num-threads', dict(
+            action  = 'store',
+            type    = int,
+            help    = "Whether to start a thread pool which events are dispatched to.",
+            default = None
+        )),
+        ('--setup', dict(
+            action  = 'store',
+            type    = str,
+            help    = "Path to a setup script to run before server starts.",
+            default = None
         ))
     )
 
+    # Supported file extensions
+    _extensions = ['.py']
+
+    def customize_applications(self, args, applications):
+        if args.index and not args.index.endswith('.html'):
+            index = args.index.split(os.path.sep)[-1]
+            for ext in self._extensions:
+                if index.endswith(ext):
+                    index = index[:-len(ext)]
+            if f'/{index}' in applications:
+                applications['/'] = applications[f'/{index}']
+        return super().customize_applications(args, applications)
+
     def customize_kwargs(self, args, server_kwargs):
         '''Allows subclasses to customize ``server_kwargs``.
 
@@ -150,9 +221,8 @@ class Serve(_BkServe):
         # Handle tranquilized functions in the supplied functions
         kwargs['extra_patterns'] = patterns = kwargs.get('extra_patterns', [])
 
-        if args.static_dirs:
-            static_dirs = parse_vars(args.static_dirs)
-            patterns += get_static_routes(static_dirs)
+        static_dirs = parse_vars(args.static_dirs) if args.static_dirs else {}
+        patterns += get_static_routes(static_dirs)
 
         files = []
         for f in args.files:
@@ -161,12 +231,20 @@ class Serve(_BkServe):
             else:
                 files.append(f)
 
-        if args.index and not args.index.endswith('.html') and not any(f.endswith(args.index) for f in files):
-            raise ValueError("The --index argument must either specify a jinja2 "
-                             "template with a .html file extension or select one "
-                             "of the applications being served as the default. "
-                             f"The specified application {args.index!r} could "
-                             "not be found.")
+        if args.index and not args.index.endswith('.html'):
+            found = False
+            for ext in self._extensions:
+                index = args.index if args.index.endswith(ext) else f'{args.index}{ext}'
+                if any(f.endswith(index) for f in files):
+                    found = True
+            if not found:
+                raise ValueError(
+                    "The --index argument must either specify a jinja2 "
+                    "template with a .html file extension or select one "
+                    "of the applications being served as the default. "
+                    f"The specified application {index!r} could not be "
+                    "found."
+                )
 
         # Handle tranquilized functions in the supplied functions
         if args.rest_provider in REST_PROVIDERS:
@@ -181,6 +259,18 @@ class Serve(_BkServe):
             for f in files:
                 watch(f)
 
+        if args.setup:
+            setup_path = args.setup
+            with open(setup_path) as f:
+                setup_source = f.read()
+            nodes = ast.parse(setup_source, os.fspath(setup_path))
+            code = compile(nodes, filename=setup_path, mode='exec', dont_inherit=True)
+            module_name = 'panel_setup_module'
+            module = ModuleType(module_name)
+            module.__dict__['__file__'] = fullpath(setup_path)
+            exec(code, module.__dict__)
+            state._setup_module = module
+
         if args.warm or args.autoreload:
             argvs = {f: args.args for f in files}
             applications = build_single_handler_applications(files, argvs)
@@ -197,13 +287,63 @@ class Serve(_BkServe):
                     with set_curdoc(doc):
                         state._on_load(None)
                     _cleanup_doc(doc)
-                    
+
+        prefix = args.prefix
+        if prefix is None:
+            prefix = ""
+        prefix = prefix.strip("/")
+        if prefix:
+            prefix = "/" + prefix
+
+        config.profiler = args.profiler
+        if args.admin:
+            from ..io.admin import admin_panel
+            from ..io.server import per_app_patterns
+            config._admin = True
+            app = Application(FunctionHandler(admin_panel))
+            unused_timeout = args.check_unused_sessions or 15000
+            app_ctx = AdminApplicationContext(app, unused_timeout=unused_timeout, url='/admin')
+            if all(not isinstance(handler, DocumentLifecycleHandler) for handler in app._handlers):
+                app.add(DocumentLifecycleHandler())
+            app_patterns = []
+            for p in per_app_patterns:
+                route = '/admin' + p[0]
+                context = {"application_context": app_ctx}
+                route = prefix + route
+                app_patterns.append((route, p[1], context))
+
+            websocket_path = None
+            for r in app_patterns:
+                if r[0].endswith("/ws"):
+                    websocket_path = r[0]
+            if not websocket_path:
+                raise RuntimeError("Couldn't find websocket path")
+            for r in app_patterns:
+                r[2]["bokeh_websocket_path"] = websocket_path
+            try:
+                import snakeviz
+                SNAKEVIZ_PATH = os.path.join(os.path.dirname(snakeviz.__file__), 'static')
+                app_patterns.append(
+                    ('/snakeviz/static/(.*)', StaticFileHandler, dict(path=SNAKEVIZ_PATH))
+                )
+            except Exception:
+                pass
+            patterns.extend(app_patterns)
+
         config.session_history = args.session_history
         if args.rest_session_info:
             pattern = REST_PROVIDERS['param'](files, 'rest')
             patterns.extend(pattern)
             state.publish('session_info', state, ['session_info'])
 
+        if args.num_threads is not None:
+            if config.nthreads is not None:
+                raise ValueError(
+                    "Supply num_threads either using the environment variable "
+                    "PANEL_NUM_THREADS or as an explicit argument, not both."
+                )
+            config.nthreads = args.num_threads
+
         if args.oauth_provider:
             config.oauth_provider = args.oauth_provider
             config.oauth_expiry = args.oauth_expiry_days
@@ -296,7 +436,7 @@ class Serve(_BkServe):
                     "CLI argument or the PANEL_COOKIE_SECRET environment "
                     "variable."
                 )
-            kwargs['auth_provider'] = OAuthProvider()
+            kwargs['auth_provider'] = OAuthProvider(error_template=args.oauth_error_template)
 
             if args.oauth_redirect_uri and config.oauth_redirect_uri:
                 raise ValueError(
