@@ -4,14 +4,21 @@ on a running bokeh server.
 """
 import asyncio
 import inspect
+import logging
 import time
+import sys
+
 import param
 
 from bokeh.io import curdoc as _curdoc
 
-from ..util import edit_readonly
+from ..util import edit_readonly, function_name
+from .logging import LOG_PERIODIC_START, LOG_PERIODIC_END
+
 from .state import state
 
+log = logging.getLogger('panel.callbacks')
+_periodic_logger = logging.getLogger(f'{__name__}.PeriodicCallback')
 
 class PeriodicCallback(param.Parameterized):
     """
@@ -67,17 +74,25 @@ class PeriodicCallback(param.Parameterized):
             self.stop()
             self.start()
 
-    async def _periodic_callback(self):
-        with edit_readonly(state):
-            state.busy = True
+    def _exec_callback(self, post=False):
+        from .state import set_curdoc
         try:
-            if inspect.isasyncgenfunction(self.callback) or inspect.iscoroutinefunction(self.callback):
-                await self.callback()
-            else:
-                self.callback()
-        finally:
-            with edit_readonly(state):
-                state.busy = False
+            with set_curdoc(self._doc):
+                cb = self.callback()
+        except Exception:
+            cb = None
+        if post:
+            self._post_callback()
+        return cb
+
+    def _post_callback(self):
+        cbname = function_name(self.callback)
+        if self._doc:
+            _periodic_logger.info(
+                LOG_PERIODIC_END, id(self._doc), cbname, self._counter
+            )
+        with edit_readonly(state):
+            state.busy = False
         self._counter += 1
         if self.timeout is not None:
             dt = (time.time() - self._start_time) * 1000
@@ -86,6 +101,31 @@ class PeriodicCallback(param.Parameterized):
         if self._counter == self.count:
             self.stop()
 
+    async def _periodic_callback(self):
+        with edit_readonly(state):
+            state.busy = True
+        cbname = function_name(self.callback)
+        if self._doc:
+            _periodic_logger.info(
+                LOG_PERIODIC_START, id(self._doc), cbname, self._counter
+            )
+        is_async = (
+            inspect.isasyncgenfunction(self.callback) or
+            inspect.iscoroutinefunction(self.callback)
+        )
+        if state._thread_pool and not is_async:
+            state._thread_pool.submit(self._exec_callback, True)
+            return
+        try:
+            cb = self._exec_callback()
+            if inspect.isawaitable(cb):
+                await cb
+        except Exception:
+            log.exception('Periodic callback failed.')
+            raise
+        finally:
+            self._post_callback()
+
     @property
     def counter(self):
         """
@@ -93,6 +133,22 @@ class PeriodicCallback(param.Parameterized):
         """
         return self._counter
 
+    async def _async_repeat(self, func):
+        """
+        Run func every interval seconds.
+
+        If func has not finished before *interval*, will run again
+        immediately when the previous iteration finished.
+        """
+        while True:
+            start = time.monotonic()
+            self._cb = asyncio.ensure_future(func())
+            await self._cb
+            timeout = (self.period/1000.) - (time.monotonic()-start)
+            if timeout > 0:
+                self._cb = asyncio.ensure_future(asyncio.sleep(timeout))
+                await self._cb
+
     def _cleanup(self, session_context):
         self.stop()
 
@@ -109,7 +165,13 @@ class PeriodicCallback(param.Parameterized):
             finally:
                 self._updating = False
         self._start_time = time.time()
-        if state.curdoc:
+        if '_pyodide' in sys.modules:
+            event_loop = asyncio.get_running_loop()
+            task = asyncio.create_task(
+                self._async_repeat(self._periodic_callback)
+            )
+            event_loop.call_soon(task)
+        elif state.curdoc:
             self._doc = state.curdoc
             self._cb = self._doc.add_periodic_callback(self._periodic_callback, self.period)
         else:
@@ -133,7 +195,9 @@ class PeriodicCallback(param.Parameterized):
                 self._updating = False
         self._counter = 0
         self._timeout = None
-        if self._doc:
+        if '_pyodide' in sys.modules:
+            self._cb.cancel()
+        elif self._doc:
             if self._doc._session_context:
                 self._doc.callbacks.remove_session_callback(self._cb)
             else:
