@@ -8,7 +8,6 @@ import sys
 
 from contextlib import contextmanager
 from collections import OrderedDict
-from six import string_types
 
 import bokeh
 import bokeh.embed.notebook
@@ -36,8 +35,7 @@ except Exception:
 from ..compiler import require_components
 from .embed import embed_state
 from .model import add_to_doc, diff
-from .resources import Bundle, Resources, _env, bundle_resources
-from .server import _server_url, _origin_url, get_server
+from .resources import PANEL_DIR, Bundle, Resources, _env, bundle_resources
 from .state import state
 
 
@@ -135,7 +133,7 @@ def html_for_render_items(docs_json, render_items, template=None, template_varia
 
     if template is None:
         template = NB_TEMPLATE_BASE
-    elif isinstance(template, string_types):
+    elif isinstance(template, str):
         template = _env.from_string("{% extends base %}\n" + template)
 
     return template.render(context)
@@ -260,8 +258,10 @@ def load_notebook(inline=True, load_timeout=5000):
     bokeh.io.notebook.curstate().output_notebook()
 
     # Publish comm manager
+    CSS = (PANEL_DIR / '_templates' / 'jupyter.css').read_text()
     JS = '\n'.join([PYVIZ_PROXY, _JupyterCommManager.js_manager, nb_mime_js])
     publish_display_data(data={LOAD_MIME: JS, 'application/javascript': JS})
+    publish_display_data(data={'text/html': f'<style>{CSS}</style>'})
 
 
 def show_server(panel, notebook_url, port):
@@ -284,6 +284,7 @@ def show_server(panel, notebook_url, port):
     server: bokeh.server.Server
     """
     from IPython.display import publish_display_data
+    from .server import _server_url, _origin_url, get_server
 
     if callable(notebook_url):
         origin = notebook_url(None)
