@@ -38,6 +38,9 @@ class Card(Column):
     header_css_classes = param.List(['card-header'], doc="""
         CSS classes to apply to the header element.""")
 
+    hide_header = param.Boolean(default=False, doc="""
+        Whether to skip rendering the header.""")
+
     title_css_classes = param.List(['card-title'], doc="""
         CSS classes to apply to the header title.""")
 
@@ -79,14 +82,15 @@ class Card(Column):
         if self.header is None:
             params = {
                 'object': '%s' % (self.title or "&#8203;"),
-                'css_classes': self.title_css_classes
+                'css_classes': self.title_css_classes,
+                'margin': (5, 0)
             }
             if self._header is not None:
                 self._header.param.set_param(**params)
                 return
             else:
                 self._header = item = HTML(
-                    sizing_mode='stretch_width', margin=(2, 5), **params
+                    sizing_mode='stretch_width', **params
                 )
         else:
             item = panel(self.header)
