@@ -6,9 +6,10 @@ from collections import defaultdict
 import param
 
 from bokeh.models import (
-    Spacer as BkSpacer, Panel as BkPanel, Tabs as BkTabs
+    Spacer as BkSpacer, Panel as BkPanel
 )
 
+from ..models import Tabs as BkTabs
 from ..viewable import Layoutable
 from .base import NamedListPanel
 
@@ -158,8 +159,7 @@ class Tabs(NamedListPanel):
                 hasattr(panel, 'child') and isinstance(panel.child, BkSpacer) and
                 panel.child.tags == ['hidden']
             )
-            if prev_hidden and not hidden and pref in rendered:
-                panel = rendered[pref]
+
             # If object has not changed, we have not toggled between
             # hidden and unhidden state or the tabs are not
             # dynamic then reuse the panel
@@ -168,7 +168,9 @@ class Tabs(NamedListPanel):
                 new_models.append(panel)
                 continue
 
-            if hidden:
+            if prev_hidden and not hidden and pref in rendered:
+                child = rendered[pref]
+            elif hidden:
                 child = BkSpacer(**{k: v for k, v in pane.param.values().items()
                                     if k in Layoutable.param and v is not None})
                 child.tags = ['hidden']
@@ -177,6 +179,7 @@ class Tabs(NamedListPanel):
                     rendered[pref] = child = pane._get_model(doc, root, model, comm)
                 except RerenderError:
                     return self._get_objects(model, current_objects[:i], doc, root, comm)
+
             panel = panels[pref] = BkPanel(
                 title=name, name=pane.name, child=child, closable=self.closable
             )
