@@ -1,186 +1,76 @@
 """
 Defines Links which allow declaring links between bokeh properties.
 """
-import param
-import weakref
+import difflib
 import sys
+import weakref
 
-import bokeh.core.properties as bp
-import param as pm
-import bokeh
+import param
 
-from bokeh.model import DataModel
-from bokeh.models import ColumnDataSource, CustomJS, Model as BkModel
+from bokeh.models import CustomJS, Model as BkModel, LayoutDOM
 
-from .config import config
-from .io.notebook import push_on_root
+from .io.datamodel import create_linked_datamodel
 from .models import ReactiveHTML
-from .reactive import Reactive, Syncable
+from .reactive import Reactive
 from .viewable import Viewable
 
 
-class Parameterized(bokeh.core.property.bases.Property):
-    """ Accept a Parameterized object.
-
-    This property only exists to support type validation, e.g. for "accepts"
-    clauses. It is not serializable itself, and is not useful to add to
-    Bokeh models directly.
-
-    """
-    def validate(self, value, detail=True):
-        super().validate(value, detail)
-
-        if isinstance(value, param.Parameterized):
-            return
-
-        msg = "" if not detail else f"expected param.Parameterized, got {value!r}"
-        raise ValueError(msg)
-
-
-
-_DATA_MODELS = weakref.WeakKeyDictionary()
-
-PARAM_MAPPING = {
-    pm.Array: lambda p, kwargs: bp.Array(bp.Any, **kwargs),
-    pm.Boolean: lambda p, kwargs: bp.Bool(**kwargs),
-    pm.CalendarDate: lambda p, kwargs: bp.Date(**kwargs),
-    pm.CalendarDateRange: lambda p, kwargs: bp.Tuple(bp.Date, bp.Date, **kwargs),
-    pm.ClassSelector: lambda p, kwargs: (
-        (bp.Instance(DataModel, **kwargs), [(Parameterized, create_linked_datamodel)])
-        if isinstance(p.class_, type) and issubclass(p.class_, param.Parameterized) else
-        bp.Any(**kwargs)
-    ),
-    pm.Color: lambda p, kwargs: bp.Color(**kwargs),
-    pm.DataFrame: lambda p, kwargs: (
-        bp.ColumnData(bp.Any, bp.Seq(bp.Any), **kwargs),
-        [(bp.PandasDataFrame, lambda x: ColumnDataSource._data_from_df(x))]
-    ),
-    pm.DateRange: lambda p, kwargs: bp.Tuple(bp.Datetime, bp.Datetime, **kwargs),
-    pm.Date: lambda p, kwargs: bp.Datetime(**kwargs),
-    pm.Dict: lambda p, kwargs: bp.Dict(bp.String, bp.Any, **kwargs),
-    pm.Event: lambda p, kwargs: bp.Bool(**kwargs),
-    pm.Integer: lambda p, kwargs: bp.Int(**kwargs),
-    pm.List: lambda p, kwargs: bp.List(bp.Any, **kwargs),
-    pm.Number: lambda p, kwargs: bp.Float(**kwargs),
-    pm.NumericTuple: lambda p, kwargs: bp.Tuple(*(bp.Float for p in p.length), **kwargs),
-    pm.Range: lambda p, kwargs: bp.Tuple(bp.Float, bp.Float, **kwargs),
-    pm.String: lambda p, kwargs: bp.String(**kwargs),
-    pm.Tuple: lambda p, kwargs: bp.Tuple(*(bp.Any for p in p.length), **kwargs),
-}
-
-
-def construct_data_model(parameterized, name=None, ignore=[], types={}):
-    """
-    Dynamically creates a Bokeh DataModel class from a Parameterized
-    object.
-
-    Arguments
-    ---------
-    parameterized: param.Parameterized
-        The Parameterized class or instance from which to create the
-        DataModel
-    name: str or None
-        Name of the dynamically created DataModel class
-    ignore: list(str)
-        List of parameters to ignore.
-    types: dict
-        A dictionary mapping from parameter name to a Parameter type,
-        making it possible to override the default parameter types.
-
-    Returns
-    -------
-    DataModel
-    """
-
-    properties = {}
-    for pname in parameterized.param:
-        if pname in ignore:
+def assert_source_syncable(source, properties):
+    for prop in properties:
+        if prop.startswith('event:'):
             continue
-        p = parameterized.param[pname]
-        if p.precedence and p.precedence < 0:
+        elif hasattr(source, 'object') and isinstance(source.object, LayoutDOM):
+            current = source.object
+            for attr in prop.split('.'):
+                if hasattr(current, attr):
+                    current = getattr(current, attr)
+                    continue
+                raise ValueError(
+                    f"Could not resolve {prop} on {source.object} model. "
+                    "Ensure you jslink an attribute that exists on the "
+                    "bokeh model."
+                )
+        elif (prop not in source.param and prop not in list(source._rename.values())):
+            matches = difflib.get_close_matches(prop, list(source.param))
+            if matches:
+                matches = f' Similar parameters include: {matches!r}'
+            else:
+                matches = ''
+            raise ValueError(
+                f"Could not jslink {prop!r} parameter (or property) "
+                f"on {type(source).__name__} object because it was not "
+                "found.{matches}."
+            )
+        elif (source._source_transforms.get(prop, False) is None or
+              source._rename.get(prop, False) is None):
+            raise ValueError(
+                f"Cannot jslink {prop!r} parameter on {type(source).__name__} "
+                "object, the parameter requires a live Python kernel "
+                "to have an effect."
+            )
+
+def assert_target_syncable(source, target, properties):
+    for k, p in properties.items():
+        if k.startswith('event:'):
             continue
-        ptype = types.get(pname, type(p))
-        prop = PARAM_MAPPING.get(ptype)
-        if isinstance(parameterized, Syncable):
-            pname = parameterized._rename.get(pname, pname)
-        if pname == 'name' or pname is None:
-            continue
-        nullable = getattr(p, 'allow_None', False)
-        kwargs = {'default': p.default, 'help': p.doc}
-        if prop is None:
-            bk_prop, accepts = bp.Any(**kwargs), []
-        else:
-            bkp = prop(p, {} if nullable else kwargs)
-            bk_prop, accepts = bkp if isinstance(bkp, tuple) else (bkp, [])
-            if nullable:
-                bk_prop = bp.Nullable(bk_prop, **kwargs)
-        for bkp, convert in accepts:
-            bk_prop = bk_prop.accepts(bkp, convert)
-        properties[pname] = bk_prop
-    name = name or parameterized.name
-    return type(name, (DataModel,), properties)
-
-
-def create_linked_datamodel(obj, root=None):
-    """
-    Creates a Bokeh DataModel from a Parameterized class or instance
-    which automatically links the parameters bi-directionally.
-
-    Arguments
-    ---------
-    obj: param.Parameterized
-       The Parameterized class to create a linked DataModel for.
-
-    Returns
-    -------
-    DataModel instance linked to the Parameterized object.
-    """
-    if isinstance(obj, type) and issubclass(obj, param.Parameterized):
-        cls = obj
-    elif isinstance(obj, param.Parameterized):
-        cls = type(obj)
-    else:
-        raise TypeError('Can only create DataModel for Parameterized class or instance.')
-    if cls in _DATA_MODELS:
-        model = _DATA_MODELS[cls]
-    else:
-        _DATA_MODELS[cls] = model = construct_data_model(obj)
-    model = model(**dict(obj.param.get_param_values()))
-    _changing = []
-
-    def cb_bokeh(attr, old, new):
-        if attr in _changing:
-            return
-        try:
-            _changing.append(attr)
-            obj.param.set_param(**{attr: new})
-        finally:
-            _changing.remove(attr)
-
-    def cb_param(*events):
-        update = {
-            event.name: event.new for event in events
-            if event.name not in _changing
-        }
-        try:
-            _changing.extend(list(update))
-            model.update(**update)
-            tags = [tag for tag in model.tags if tag.startswith('__ref:')]
-            if root:
-                push_on_root(root.ref['id'])
-            elif tags:
-                ref = tags[0].split('__ref:')[-1]
-                push_on_root(ref)
-        finally:
-            for attr in update:
-                _changing.remove(attr)
-
-    for p in obj.param:
-        model.on_change(p, cb_bokeh)
-
-    obj.param.watch(cb_param, list(obj.param))
-
-    return model
+        elif p not in target.param and p not in list(target._rename.values()):
+            matches = difflib.get_close_matches(p, list(target.param))
+            if matches:
+                matches = ' Similar parameters include: %r' % matches
+            else:
+                matches = ''
+            raise ValueError(
+                f"Could not jslink {p!r} parameter (or property) "
+                f"on {type(source).__name__} object because it was not "
+                "found. Similar parameters include: {matches}"
+            )
+        elif (target._source_transforms.get(p, False) is None or
+              target._rename.get(p, False) is None):
+            raise ValueError(
+                f"Cannot jslink {k!r} parameter on {type(source).__name__} "
+                f"object to {p!r} parameter on {type(target).__name__}. "
+                "It requires a live Python kernel to have an effect."
+            )
 
 
 class Callback(param.Parameterized):
@@ -225,10 +115,12 @@ class Callback(param.Parameterized):
         if self.source in self.registry:
             links = self.registry[self.source]
             params = {
-                k: v for k, v in self.param.get_param_values() if k != 'name'}
+                k: v for k, v in self.param.values().items() if k != 'name'
+            }
             for link in links:
                 link_params = {
-                    k: v for k, v in link.param.get_param_values() if k != 'name'}
+                    k: v for k, v in link.param.values().items() if k != 'name'
+                }
                 if not hasattr(link, 'target'):
                     pass
                 elif (type(link) is type(self) and link.source is self.source
@@ -270,7 +162,12 @@ class Callback(param.Parameterized):
 
         arg_overrides = {}
         if 'holoviews' in sys.modules:
+            from holoviews.core.dimension import Dimensioned
             from .pane.holoviews import HoloViews, generate_panel_bokeh_map
+            found = [
+                (link, src, tgt) for (link, src, tgt) in found
+                if not (isinstance(src, Dimensioned) or isinstance(tgt, Dimensioned))
+            ]
             hv_views = root_view.select(HoloViews)
             map_hve_bk = generate_panel_bokeh_map(root_model, hv_views)
             for src in linkable:
@@ -346,10 +243,12 @@ class Link(Callback):
         if self.source in self.registry:
             links = self.registry[self.source]
             params = {
-                k: v for k, v in self.param.get_param_values() if k != 'name'}
+                k: v for k, v in self.param.values().items() if k != 'name'
+            }
             for link in links:
                 link_params = {
-                    k: v for k, v in link.param.get_param_values() if k != 'name'}
+                    k: v for k, v in link.param.values().items() if k != 'name'
+                }
                 if (type(link) is type(self) and link.source is self.source
                     and link.target is self.target and params == link_params):
                     return
@@ -437,12 +336,13 @@ class CallbackGenerator(object):
         return model
 
     def _init_callback(self, root_model, link, source, src_spec, target, tgt_spec, code):
-        references = {k: v for k, v in link.param.get_param_values()
+        references = {k: v for k, v in link.param.values().items()
                       if k not in ('source', 'target', 'name', 'code', 'args')}
 
         src_model = self._resolve_model(root_model, source, src_spec[0])
         ref = root_model.ref['id']
-        link_id = id(link)
+
+        link_id = (id(link), src_spec, tgt_spec)
         if (any(link_id in cb.tags for cbs in src_model.js_property_callbacks.values() for cb in cbs) or
             any(link_id in cb.tags for cbs in src_model.js_event_callbacks.values() for cb in cbs)):
             # Skip registering callback if already registered
@@ -734,6 +634,7 @@ class JSLinkCallbackGenerator(JSCallbackGenerator):
         else:
             tgt_transform = 'value'
         if tgt_spec == 'loading':
+            from .config import config
             return self._loading_link_template.format(
                 src_attr=src_spec, src_transform=src_transform,
                 loading_spinner=config.loading_spinner
