@@ -11,9 +11,10 @@ from .ipywidget import IPyWidget # noqa
 from .layout import Card # noqa
 from .location import Location # noqa
 from .markup import JSON, HTML # noqa
+from .reactive_html import ReactiveHTML # noqa
 from .state import State # noqa
-from .tabulator import DataTabulator # noqa
+from .tabs import Tabs # noqa
 from .trend import TrendIndicator # noqa
 from .widgets import ( # noqa
-    Audio, FileDownload, Player, Progress, SingleSelect, Video, VideoStream
+    Audio, CustomSelect, FileDownload, Player, Progress, SingleSelect, Video, VideoStream
 )
