@@ -2,6 +2,7 @@ export {AcePlot} from "./ace"
 export {Audio} from "./audio"
 export {Card} from "./card"
 export {CommManager} from "./comm_manager"
+export {CustomSelect} from "./customselect"
 export {DataTabulator} from "./tabulator"
 export {DatetimePicker} from "./datetime_picker"
 export {DeckGLPlot} from "./deckgl"
@@ -10,6 +11,7 @@ export {HTML} from "./html"
 export {IDOM} from "./idom"
 export {IPyWidget} from "./ipywidget"
 export {JSON} from "./json"
+export {JSONEditor} from "./json_editor"
 export {FileDownload} from "./file_download"
 export {KaTeX} from "./katex"
 export {Location} from "./location"
@@ -18,9 +20,13 @@ export {Perspective} from "./perspective"
 export {Player} from "./player"
 export {PlotlyPlot} from "./plotly"
 export {Progress} from "./progress"
+export {QuillInput} from "./quill"
+export {ReactiveHTML} from "./reactive_html"
 export {SingleSelect} from "./singleselect"
 export {SpeechToText} from "./speech_to_text"
 export {State} from "./state"
+export {Tabs} from "./tabs"
+export {Terminal} from "./terminal"
 export {TextToSpeech} from "./text_to_speech"
 export {TrendIndicator} from "./trend"
 export {VegaPlot} from "./vega"
