@@ -1,4 +1,3 @@
-# coding: utf-8
 """
 Defines custom VTKPlot bokeh model to render VTK objects.
 """
@@ -13,7 +12,7 @@ from bokeh.models import HTMLBox, Model, ColorMapper
 from ..io.resources import bundled_files
 from ..util import classproperty
 
-vtk_cdn = "https://unpkg.com/vtk.js@14.16.4/dist/vtk.js"
+vtk_cdn = "https://unpkg.com/vtk.js@20.0.1/vtk.js"
 
 class VTKAxes(Model):
     """
@@ -135,6 +134,8 @@ class VTKVolumePlot(AbstractVTKPlot):
 
     mapper = Dict(String, Any)
 
+    nan_opacity = Float(default=1)
+
     render_background = String(default='#52576e')
 
     rescale = Bool(default=False)
