@@ -5,7 +5,7 @@ objects and their widgets and support for Links
 import sys
 
 from collections import OrderedDict, defaultdict
-from distutils.version import LooseVersion
+from packaging.version import Version
 from functools import partial
 
 import param
@@ -24,8 +24,17 @@ from .plotly import Plotly
 
 class HoloViews(PaneBase):
     """
-    HoloViews panes render any HoloViews object to a corresponding
-    Bokeh model while respecting the currently selected backend.
+    `HoloViews` panes render any `HoloViews` object using the
+    currently selected backend ('bokeh' (default), 'matplotlib' or 'plotly').
+
+    To be able to use the `plotly` backend you must add `plotly` to
+    `pn.extension`.
+    
+    Reference: https://panel.holoviz.org/reference/panes/HoloViews.html
+
+    :Example:
+
+    >>> HoloViews(some_holoviews_object)
     """
 
     backend = param.ObjectSelector(
@@ -37,7 +46,7 @@ class HoloViews(PaneBase):
         Whether to center the plot.""")
 
     linked_axes = param.Boolean(default=True, doc="""
-        Whether to use link the axes of bokeh plots inside this pane
+        Whether to link the axes of bokeh plots inside this pane
         across a panel layout.""")
 
     renderer = param.Parameter(default=None, doc="""
@@ -257,7 +266,7 @@ class HoloViews(PaneBase):
         else:
             self._responsive_content = False
 
-        kwargs = {p: v for p, v in self.param.get_param_values()
+        kwargs = {p: v for p, v in self.param.values().items()
                   if p in Layoutable.param and p != 'name'}
         child_pane = self._get_pane(backend, state, **kwargs)
         self._update_plot(plot, child_pane)
@@ -303,7 +312,7 @@ class HoloViews(PaneBase):
                 renderer = renderer.instance(**params)
 
         kwargs = {'margin': self.margin}
-        if backend == 'bokeh' or LooseVersion(str(hv.__version__)) >= str('1.13.0'):
+        if backend == 'bokeh' or Version(str(hv.__version__)) >= Version('1.13.0'):
             kwargs['doc'] = doc
             kwargs['root'] = root
             if comm:
@@ -335,11 +344,28 @@ class HoloViews(PaneBase):
         from holoviews.plotting.plot import Plot
         return isinstance(obj, Dimensioned) or isinstance(obj, Plot)
 
+    def jslink(self, target, code=None, args=None, bidirectional=False, **links):
+        if links and code:
+            raise ValueError('Either supply a set of properties to '
+                             'link as keywords or a set of JS code '
+                             'callbacks, not both.')
+        elif not links and not code:
+            raise ValueError('Declare parameters to link or a set of '
+                             'callbacks, neither was defined.')
+        if args is None:
+            args = {}
+
+        from ..links import Link
+        return Link(self, target, properties=links, code=code, args=args,
+                    bidirectional=bidirectional)
+
+    jslink.__doc__ = PaneBase.jslink.__doc__
+
     @classmethod
     def widgets_from_dimensions(cls, object, widget_types=None, widgets_type='individual'):
         from holoviews.core import Dimension, DynamicMap
         from holoviews.core.options import SkipRendering
-        from holoviews.core.util import isnumeric, unicode, datetime_types, unique_iterator
+        from holoviews.core.util import isnumeric, datetime_types, unique_iterator
         from holoviews.core.traversal import unique_dimkeys
         from holoviews.plotting.plot import Plot, GenericCompositePlot
         from holoviews.plotting.util import get_dynamic_mode
@@ -401,7 +427,7 @@ class HoloViews(PaneBase):
             elif dim.name in widget_types:
                 widget = widget_types[dim.name]
                 if isinstance(widget, Widget):
-                    widget.param.set_param(**kwargs)
+                    widget.param.update(**kwargs)
                     if not widget.name:
                         widget.name = dim.label
                     widgets.append(widget)
@@ -421,7 +447,7 @@ class HoloViews(PaneBase):
             if vals:
                 if all(isnumeric(v) or isinstance(v, datetime_types) for v in vals) and len(vals) > 1:
                     vals = sorted(vals)
-                    labels = [unicode(dim.pprint_value(v)) for v in vals]
+                    labels = [str(dim.pprint_value(v)) for v in vals]
                     options = OrderedDict(zip(labels, vals))
                     widget_type = widget_type or DiscreteSlider
                 else:
@@ -478,14 +504,14 @@ class Interactive(PaneBase):
             self._layout_panel = None
         else:
             self._layout_panel = self.object.layout()
-            self._layout_panel.param.set_param(**{
+            self._layout_panel.param.update(**{
                 p: getattr(self, p) for p in Layoutable.param if p != 'name'
             })
 
     def _update_layout_properties(self, *events):
         if self._layout_panel is None:
             return
-        self._layout_panel.param.set_param(**{e.name: e.new for e in events})
+        self._layout_panel.param.update(**{e.name: e.new for e in events})
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
         if root is None:
