@@ -2,12 +2,11 @@
 Contains Image panes including renderers for PNG, SVG, GIF and JPG
 file types.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import base64
 
+from pathlib import PurePath
+
 from io import BytesIO
-from six import string_types
 
 import param
 
@@ -15,13 +14,76 @@ from .markup import escape, DivPaneBase
 from ..util import isfile, isurl
 
 
-class ImageBase(DivPaneBase):
+class FileBase(DivPaneBase):
+
+    embed = param.Boolean(default=True, doc="""
+        Whether to embed the file as base64.""")
+
+    _rerender_params = ['embed', 'object', 'style', 'width', 'height']
+
+    __abstract = True
+
+    def __init__(self, object=None, **params):
+        if isinstance(object, PurePath):
+            object = str(object)
+        super().__init__(object=object, **params)
+
+    def _type_error(self, object):
+        if isinstance(object, str):
+            raise ValueError("%s pane cannot parse string that is not a filename "
+                             "or URL." % type(self).__name__)
+        super()._type_error(object)
+
+    @classmethod
+    def applies(cls, obj):
+        filetype = cls.filetype
+        if hasattr(obj, '_repr_{}_'.format(filetype)):
+            return True
+        if isinstance(obj, PurePath):
+            obj = str(obj.absolute())
+        if isinstance(obj, str):
+            if isfile(obj) and obj.endswith('.'+filetype):
+                return True
+            if isurl(obj, [cls.filetype]):
+                return True
+            elif isurl(obj, None):
+                return 0
+        elif isinstance(obj, bytes):
+            try:
+                cls._imgshape(obj)
+                return True
+            except Exception:
+                return False
+        if hasattr(obj, 'read'):  # Check for file like object
+            return True
+        return False
+
+    def _data(self):
+        if hasattr(self.object, '_repr_{}_'.format(self.filetype)):
+            return getattr(self.object, '_repr_' + self.filetype + '_')()
+        if isinstance(self.object, str):
+            if isfile(self.object):
+                with open(self.object, 'rb') as f:
+                    return f.read()
+        elif isinstance(self.object, bytes):
+            return self.object
+        if hasattr(self.object, 'read'):
+            if hasattr(self.object, 'seek'):
+                self.object.seek(0)
+            return self.object.read()
+        if isurl(self.object, None):
+            import requests
+            r = requests.request(url=self.object, method='GET')
+            return r.content
+
+
+class ImageBase(FileBase):
     """
     Encodes an image as base64 and wraps it in a Bokeh Div model.
     This is an abstract base class that needs the image type
     to be specified and specific code for determining the image shape.
 
-    The imgtype determines the filetype, extension, and MIME type for
+    The filetype determines the filetype, extension, and MIME type for
     this image. Each image type (png,jpg,gif) has a base class that
     supports anything with a `_repr_X_` method (where X is `png`,
     `gif`, etc.), a local file with the given file extension, or a
@@ -37,10 +99,7 @@ class ImageBase(DivPaneBase):
         A link URL to make the image clickable and link to some other
         website.""")
 
-    embed = param.Boolean(default=True, doc="""
-        Whether to embed the image as base64.""")
-
-    imgtype = 'None'
+    filetype = 'None'
 
     _rerender_params = ['alt_text', 'link_url', 'embed', 'object', 'style', 'width', 'height']
 
@@ -48,60 +107,22 @@ class ImageBase(DivPaneBase):
 
     __abstract = True
 
-    @classmethod
-    def applies(cls, obj):
-        imgtype = cls.imgtype
-        if hasattr(obj, '_repr_{}_'.format(imgtype)):
-            return True
-        if isinstance(obj, string_types):
-            if isfile(obj) and obj.endswith('.'+imgtype):
-                return True
-            if isurl(obj, [cls.imgtype]):
-                return True
-            elif isurl(obj, None):
-                return 0
-        if hasattr(obj, 'read'):  # Check for file like object
-            return True
-        return False
-
-    def _type_error(self, object):
-        if isinstance(object, string_types):
-            raise ValueError("%s pane cannot parse string that is not a filename "
-                             "or URL." % type(self).__name__)
-        super(ImageBase, self)._type_error(object)
-
-    def _img(self):
-        if hasattr(self.object, '_repr_{}_'.format(self.imgtype)):
-            return getattr(self.object, '_repr_' + self.imgtype + '_')()
-        if isinstance(self.object, string_types):
-            if isfile(self.object):
-                with open(self.object, 'rb') as f:
-                    return f.read()
-        if hasattr(self.object, 'read'):
-            if hasattr(self.object, 'seek'):
-                self.object.seek(0)
-            return self.object.read()
-        if isurl(self.object, None):
-            import requests
-            r = requests.request(url=self.object, method='GET')
-            return r.content
-
     def _b64(self):
-        data = self._img()
+        data = self._data()
         if not isinstance(data, bytes):
             data = data.encode('utf-8')
         b64 = base64.b64encode(data).decode("utf-8")
-        return "data:image/"+self.imgtype+f";base64,{b64}"
+        return "data:image/"+self.filetype+f";base64,{b64}"
 
     def _imgshape(self, data):
         """Calculate and return image width,height"""
         raise NotImplementedError
 
     def _get_properties(self):
-        p = super(ImageBase, self)._get_properties()
+        p = super()._get_properties()
         if self.object is None:
             return dict(p, text='<img></img>')
-        data = self._img()
+        data = self._data()
         if not isinstance(data, bytes):
             data = base64.b64decode(data)
         width, height = self._imgshape(data)
@@ -118,7 +139,7 @@ class ImageBase(DivPaneBase):
             src = self.object
         else:
             b64 = base64.b64encode(data).decode("utf-8")
-            src = "data:image/"+self.imgtype+";base64,{b64}".format(b64=b64)
+            src = "data:image/"+self.filetype+";base64,{b64}".format(b64=b64)
 
         smode = self.sizing_mode
         if smode in ['fixed', None]:
@@ -145,8 +166,23 @@ class ImageBase(DivPaneBase):
 
 
 class PNG(ImageBase):
+    """
+    The `PNG` pane embeds a .png image file in a panel if provided a local
+    path, or will link to a remote image if provided a URL.
+    
+    Reference: https://panel.holoviz.org/reference/panes/PNG.html
 
-    imgtype = 'png'
+    :Example:
+
+    >>> PNG(
+    ...     'https://panel.holoviz.org/_static/logo_horizontal.png',
+    ...     alt_text='The Panel Logo',
+    ...     link_url='https://panel.holoviz.org/index.html',
+    ...     width=500
+    ... )
+    """
+
+    filetype = 'png'
 
     @classmethod
     def _imgshape(cls, data):
@@ -156,8 +192,23 @@ class PNG(ImageBase):
 
 
 class GIF(ImageBase):
+    """
+    The `GIF` pane embeds a .gif image file in a panel if provided a local
+    path, or will link to a remote image if provided a URL.
+    
+    Reference: https://panel.holoviz.org/reference/panes/GIF.html
 
-    imgtype = 'gif'
+    :Example:
+
+    >>> GIF(
+    ...     'https://upload.wikimedia.org/wikipedia/commons/b/b1/Loading_icon.gif',
+    ...     alt_text='A loading spinner',
+    ...     link_url='https://commons.wikimedia.org/wiki/File:Loading_icon.gif',
+    ...     width=500
+    ... )
+    """
+
+    filetype = 'gif'
 
     @classmethod
     def _imgshape(cls, data):
@@ -166,9 +217,50 @@ class GIF(ImageBase):
         return int(w), int(h)
 
 
-class JPG(ImageBase):
+class ICO(ImageBase):
+    """
+    The `ICO` pane embeds an .ico image file in a panel if provided a local
+    path, or will link to a remote image if provided a URL.
+    
+    Reference: https://panel.holoviz.org/reference/panes/ICO.html
 
-    imgtype = 'jpg'
+    :Example:
+
+    >>> ICO(
+    ...     some_url,
+    ...     alt_text='An .ico file',
+    ...     link_url='https://en.wikipedia.org/wiki/ICO_(file_format)',
+    ...     width=50
+    ...
+    """
+
+    filetype = 'ico'
+
+    @classmethod
+    def _imgshape(cls, data):
+        import struct
+        w, h = struct.unpack("<BB" , data[6:8])
+        return int(w or 256), int(h or 256)
+
+
+class JPG(ImageBase):
+    """
+    The `JPG` pane embeds a .jpg or .jpeg image file in a panel if provided a
+    local path, or will link to a remote image if provided a URL.
+    
+    Reference: https://panel.holoviz.org/reference/panes/JPG.html
+
+    :Example:
+
+    >>> JPG(
+    ...     'https://www.gstatic.com/webp/gallery/4.sm.jpg',
+    ...     alt_text='A nice tree',
+    ...     link_url='https://en.wikipedia.org/wiki/JPEG',
+    ...     width=500
+    ... )
+    """
+
+    filetype = 'jpg'
 
     @classmethod
     def _imgshape(cls, data):
@@ -190,34 +282,49 @@ class JPG(ImageBase):
 
 
 class SVG(ImageBase):
+    """
+    The `SVG` pane embeds a .svg image file in a panel if provided a
+    local path, or will link to a remote image if provided a URL.
+    
+    Reference: https://panel.holoviz.org/reference/panes/SVG.html
+
+    :Example:
+
+    >>> SVG(
+    ...     'https://upload.wikimedia.org/wikipedia/commons/6/6b/Bitmap_VS_SVG.svg',
+    ...     alt_text='A gif vs svg comparison',
+    ...     link_url='https://en.wikipedia.org/wiki/SVG',
+    ...     width=300, height=400
+    ... )
+    """
 
     encode = param.Boolean(default=False, doc="""
         Whether to enable base64 encoding of the SVG, base64 encoded
         SVGs do not support links.""")
 
-    imgtype = 'svg'
+    filetype = 'svg'
 
     _rerender_params = ImageBase._rerender_params + ['encode']
 
     @classmethod
     def applies(cls, obj):
-        return (super(SVG, cls).applies(obj) or
-                (isinstance(obj, string_types) and obj.lstrip().startswith('<svg')))
+        return (super().applies(obj) or
+                (isinstance(obj, str) and obj.lstrip().startswith('<svg')))
 
     def _type_error(self, object):
-        if isinstance(object, string_types):
+        if isinstance(object, str):
             raise ValueError("%s pane cannot parse string that is not a filename, "
                              "URL or a SVG XML contents." % type(self).__name__)
-        super(SVG, self)._type_error(object)
+        super()._type_error(object)
 
-    def _img(self):
-        if (isinstance(self.object, string_types) and
+    def _data(self):
+        if (isinstance(self.object, str) and
             self.object.lstrip().startswith('<svg')):
             return self.object
-        return super(SVG, self)._img()
+        return super()._data()
 
     def _b64(self):
-        data = self._img()
+        data = self._data()
         if not isinstance(data, bytes):
             data = data.encode('utf-8')
         b64 = base64.b64encode(data).decode("utf-8")
@@ -230,7 +337,7 @@ class SVG(ImageBase):
         p = super(ImageBase, self)._get_properties()
         if self.object is None:
             return dict(p, text='<img></img>')
-        data = self._img()
+        data = self._data()
         width, height = self._imgshape(data)
         if not isinstance(data, bytes):
             data = data.encode('utf-8')
@@ -244,3 +351,37 @@ class SVG(ImageBase):
         else:
             html = data.decode("utf-8")
         return dict(p, width=width, height=height, text=escape(html))
+
+
+class PDF(FileBase):
+    """
+    The `PDF` pane embeds a .pdf image file in a panel if provided a
+    local path, or will link to a remote image if provided a URL.
+    
+    Reference: https://panel.holoviz.org/reference/panes/PDF.html
+
+    :Example:
+
+    >>> PDF(
+    ...     'https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf',
+    ...     width=300, height=410
+    ... )
+    """
+
+    filetype = 'pdf'
+
+    def _get_properties(self):
+        p = super()._get_properties()
+        if self.object is None:
+            return dict(p, text='<embed></embed>')
+        if self.embed:
+            data = self._data()
+            if not isinstance(data, bytes):
+                data = data.encode('utf-8')
+            base64_pdf = base64.b64encode(data).decode("utf-8")
+            src = f"data:application/pdf;base64,{base64_pdf}"
+        else:
+            src = self.object
+        w, h = self.width or '100%', self.height or '100%'
+        html = f'<embed src="{src}" width={w!r} height={h!r} type="application/pdf">'
+        return dict(p, text=escape(html))
