@@ -5,7 +5,6 @@ import os
 
 from base64 import b64encode
 from io import BytesIO
-from six import string_types
 
 import numpy as np
 import param
@@ -35,6 +34,13 @@ class _MediaBase(PaneBase):
     volume = param.Number(default=None, bounds=(0, 100), doc="""
         The volume of the media player.""")
 
+    autoplay = param.Boolean(default=False, doc="""
+        When True, it specifies that the output will play automatically. 
+        In Chromium browsers this requires the user to click play once.""")
+
+    muted = param.Boolean(default=False, doc="""
+        When True, it specifies that the output should be muted.""")
+
     _default_mime = None
 
     _formats = []
@@ -43,13 +49,15 @@ class _MediaBase(PaneBase):
 
     _rename = {'name': None, 'sample_rate': None, 'object': 'value'}
 
+    _rerender_params = []
+
     _updates = True
 
     __abstract = True
 
     @classmethod
     def applies(cls, obj):
-        if isinstance(obj, string_types):
+        if isinstance(obj, str):
             if isfile(obj) and any(obj.endswith('.'+fmt) for fmt in cls._formats):
                 return True
             if isurl(obj, cls._formats):
@@ -60,8 +68,6 @@ class _MediaBase(PaneBase):
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
         props = self._process_param_change(self._init_params())
-        if self.object is not None:
-            props['value'] = self.object
         model = self._bokeh_model(**props)
         if root is None:
             root = model
@@ -75,10 +81,17 @@ class _MediaBase(PaneBase):
         wavfile.write(buffer, self.sample_rate, data)
         return buffer
 
+    def _process_property_change(self, msg):
+        msg = super()._process_property_change(msg)
+        if 'js_property_callbacks' in msg:
+            del msg['js_property_callbacks']
+        return msg
+
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         if 'value' in msg:
             value = msg['value']
+            fmt = self._default_mime
             if isinstance(value, np.ndarray):
                 fmt = 'wav'
                 buffer = self._from_numpy(value)
@@ -90,20 +103,33 @@ class _MediaBase(PaneBase):
                 data = b64encode(data)
             elif value.lower().startswith('http'):
                 return msg
-            elif not value:
-                data, fmt = b'', self._default_mime
+            elif not value or value == f'data:{self._media_type}/{fmt};base64,':
+                data = b''
             else:
-                raise ValueError('Object should be either path to a sound file or numpy array')
-            template = 'data:audio/{mime};base64,{data}'
-            msg['value'] = template.format(data=data.decode('utf-8'),
-                                           mime=fmt)
-            
+                raise ValueError(f'Object should be either path to a {self._media_type} file or numpy array.')
+            msg['value'] = f"data:{self._media_type}/{fmt};base64,{data.decode('utf-8')}"
         return msg
 
 
 class Audio(_MediaBase):
+    """
+    The `Audio` pane displays an audio player given a local or remote audio
+    file or numpy array.
+    
+    The pane also allows access and control over the player state including
+    toggling of playing/paused and loop state, the current time, and the
+    volume.
+    
+    The audio player supports ogg, mp3, and wav files as well as numpy arrays.
+    
+    Reference: https://panel.holoviz.org/reference/panes/Audio.html
 
-    object = param.ClassSelector(default='', class_=(string_types + (np.ndarray,)),
+    :Example:
+
+    >>> Audio('http://ccrma.stanford.edu/~jos/mp3/pno-cs.mp3', name='Audio')
+    """
+
+    object = param.ClassSelector(default='', class_=(str, np.ndarray,),
                                  allow_None=True, doc="""
         The audio file either local or remote.""")
 
@@ -125,6 +151,26 @@ class Audio(_MediaBase):
 
 
 class Video(_MediaBase):
+    """
+    The `Video` Pane displays a video player given a local or remote video
+    file.
+    
+    The widget also allows access and control over the player state including
+    toggling of playing/paused and loop state, the current time, and the
+    volume.
+    
+    Depending on the browser the video player supports mp4, webm, and ogg
+    containers and a variety of codecs.
+    
+    Reference: https://panel.holoviz.org/reference/panes/Video.html
+
+    :Example:
+
+    >>> Video(
+    ...     'https://file-examples-com.github.io/uploads/2017/04/file_example_MP4_640_3MG.mp4',
+    ...     width=640, height=360, loop=True
+    ... )
+    """
 
     _bokeh_model = _BkVideo
 
