@@ -9,6 +9,17 @@ from .base import ReplacementPane
 
 
 class Streamz(ReplacementPane):
+    """
+    The `Streamz` pane renders streamz `Stream` objects emitting arbitrary
+    objects, unlike the DataFrame pane which specifically handles streamz
+    DataFrame and Series objects and exposes various formatting objects.
+    
+    Reference: https://panel.holoviz.org/reference/panes/Streamz.html
+
+    :Example:
+
+    >>> Streamz(some_streamz_stream_object, always_watch=True)
+    """
 
     always_watch = param.Boolean(default=False, doc="""
         Whether to watch even when not displayed.""")
