@@ -1,5 +1,7 @@
 import sys
 
+from functools import partial
+
 import param
 import numpy as np
 
@@ -7,7 +9,7 @@ from bokeh.models import ColumnDataSource
 from pyviz_comms import JupyterComm
 
 from ..viewable import Layoutable
-from ..util import lazy_load, string_types
+from ..util import lazy_load
 from .base import PaneBase
 
 
@@ -27,24 +29,110 @@ def ds_as_cds(dataset):
     return data
 
 
+_containers = ['hconcat', 'vconcat', 'layer']
+
+def _isin(obj, attr):
+    if isinstance(obj, dict):
+        return attr in obj
+    else:
+        return hasattr(obj, attr)
+
+
+def _get_selections(obj):
+    selections = []
+    if _isin(obj, 'selection'):
+        try:
+            selections += obj['selection']
+        except TypeError:
+            pass
+    for c in _containers:
+        if _isin(obj, c):
+            for subobj in obj[c]:
+                selections += _get_selections(subobj)
+    return selections
+
+
+class Selection(param.Parameterized):
+    """
+    The Events object is dynamically updated to allow listening to
+    selection events.
+    """
+
+
 class Vega(PaneBase):
     """
-    Vega panes allow rendering Vega plots and traces.
+    The Vega pane renders Vega-lite based plots (including those from Altair)
+    inside a panel.
+    
+    Note
+    
+    - to use the `Vega` pane, the Panel `extension` has to be
+    loaded with 'vega' as an argument to ensure that vega.js is initialized.
+    - it supports selection events
+    - it optimizes the plot rendering by using binary serialization for any
+    array data found on the Vega/Altair object, providing huge speedups over
+    the standard JSON serialization employed by Vega natively.
+    
+    Reference: https://panel.holoviz.org/reference/panes/Vega.html
 
-    For efficiency any array objects found inside a Figure are added
-    to a ColumnDataSource which allows using binary transport to sync
-    the figure on bokeh server and via Comms.
+    :Example:
+
+    >>> pn.extension('vega')
+    >>> Vega(some_vegalite_dict_or_altair_object, height=240)
     """
 
+    debounce = param.ClassSelector(default=20, class_=(int, dict), doc="""
+        Declares the debounce time in milliseconds either for all
+        events or if a dictionary is provided for individual events.""")
+
     margin = param.Parameter(default=(5, 5, 30, 5), doc="""
         Allows to create additional space around the component. May
         be specified as a two-tuple of the form (vertical, horizontal)
         or a four-tuple (top, right, bottom, left).""")
 
+    selection = param.ClassSelector(class_=Selection, doc="""
+        The Selection object reflects any selections available on the
+        supplied vega plot into Python.""")
+
+    show_actions = param.Boolean(default=False, doc="""
+        Whether to show Vega actions.""")
+
+    theme = param.ObjectSelector(default=None, allow_None=True, objects=[
+        'excel', 'ggplot2', 'quartz', 'vox', 'fivethirtyeight', 'dark',
+        'latimes', 'urbaninstitute', 'googlecharts'])
+
     priority = 0.8
 
+    _rename = {'selection': None, 'debounce': None}
+
     _updates = True
 
+    def __init__(self, object=None, **params):
+        super().__init__(object, **params)
+        self.param.watch(self._update_selections, ['object'])
+        self._update_selections()
+
+    @property
+    def _selections(self):
+        return _get_selections(self.object)
+
+    @property
+    def _throttle(self):
+        default = self.param.debounce.default
+        if isinstance(self.debounce, dict):
+            throttle = {
+                sel: self.debounce.get(sel, default)
+                for sel in self._selections
+            }
+        else:
+            throttle = {sel: self.debounce or default for sel in self._selections}    
+        return throttle
+
+    def _update_selections(self, *args):
+        self.selection = Selection()
+        for e in self._selections:
+            self.selection.param._add_parameter(e, param.Dict())
+
     @classmethod
     def is_altair(cls, obj):
         if 'altair' in sys.modules:
@@ -102,7 +190,6 @@ class Vega(PaneBase):
                 if 'values' in d:
                     sources[d['name']] = ColumnDataSource(data=ds_as_cds(d.pop('values')))
 
-
     @classmethod
     def _get_dimensions(cls, json, props):
         if json is None:
@@ -122,7 +209,7 @@ class Vega(PaneBase):
                 view['height'] = size_config[h]
 
         for p in ('width', 'height'):
-            if p not in view or isinstance(view[p], string_types):
+            if p not in view or isinstance(view[p], str):
                 continue
             if props.get(p) is None or p in view and props.get(p) < view[p]:
                 v = view[p]
@@ -137,6 +224,9 @@ class Vega(PaneBase):
         elif responsive_height:
             props['sizing_mode'] = 'stretch_height'
 
+    def _process_event(self, event):
+        self.selection.param.update(**{event.data['type']: event.data['value']})
+
     def _get_model(self, doc, root=None, parent=None, comm=None):
         VegaPlot = lazy_load('panel.models.vega', 'VegaPlot', isinstance(comm, JupyterComm), root)
         sources = {}
@@ -147,7 +237,14 @@ class Vega(PaneBase):
             self._get_sources(json, sources)
         props = self._process_param_change(self._init_params())
         self._get_dimensions(json, props)
-        model = VegaPlot(data=json, data_sources=sources, **props)
+        model = VegaPlot(
+            data=json, data_sources=sources, events=self._selections,
+            throttle=self._throttle, **props
+        )
+        if comm:
+            model.on_event('vega_event', self._comm_event)
+        else:
+            model.on_event('vega_event', partial(self._server_event, doc))
         if root is None:
             root = model
         self._models[root.ref['id']] = (model, parent)
@@ -161,6 +258,8 @@ class Vega(PaneBase):
             self._get_sources(json, model.data_sources)
         props = {p : getattr(self, p) for p in list(Layoutable.param)
                  if getattr(self, p) is not None}
+        props['throttle'] = self._throttle
+        props['events'] = self._selections
         self._get_dimensions(json, props)
         props['data'] = json
         model.update(**props)
