@@ -6,10 +6,10 @@ models rendered on the frontend.
 
 import difflib
 import datetime as dt
+import logging
 import re
 import sys
 import textwrap
-import threading
 
 from collections import Counter, defaultdict, namedtuple
 from functools import partial
@@ -18,22 +18,22 @@ import bleach
 import numpy as np
 import param
 
-from bokeh.models import LayoutDOM
 from bokeh.model import DataModel
 from param.parameterized import ParameterizedMetaclass, Watcher
-from tornado import gen
+from pprint import pformat
 
-from .config import config
+from .io.document import unlocked
 from .io.model import hold
 from .io.notebook import push
-from .io.server import unlocked
-from .io.state import state
+from .io.state import set_curdoc, state
 from .models.reactive_html import (
     ReactiveHTML as _BkReactiveHTML, ReactiveHTMLParser
 )
 from .util import edit_readonly, escape, updating
 from .viewable import Layoutable, Renderable, Viewable
 
+log = logging.getLogger('panel.reactive')
+
 LinkWatcher = namedtuple("Watcher", Watcher._fields+('target', 'links', 'transformed', 'bidirectional_watcher'))
 
 
@@ -58,6 +58,9 @@ class Syncable(Renderable):
     # Timeout before the first event is processed
     _debounce = 50
 
+    # Property changes which should not be debounced
+    _priority_changes = []
+
     # Any parameters that require manual updates handling for the models
     # e.g. parameters which affect some sub-model
     _manual_params = []
@@ -146,7 +149,7 @@ class Syncable(Renderable):
         return [p for p in self.param if p not in self._manual_params+ignored]
 
     def _init_params(self):
-        return {k: v for k, v in self.param.get_param_values()
+        return {k: v for k, v in self.param.values().items()
                 if k in self._synced_params and v is not None}
 
     def _link_params(self):
@@ -156,6 +159,7 @@ class Syncable(Renderable):
             self._callbacks.append(watcher)
 
     def _link_props(self, model, properties, doc, root, comm=None):
+        from .config import config
         ref = root.ref['id']
         if config.embed:
             return
@@ -212,14 +216,22 @@ class Syncable(Renderable):
             doc.add_next_tick_callback(cb)
 
     def _update_model(self, events, msg, root, model, doc, comm):
-        self._changing[root.ref['id']] = [
+        ref = root.ref['id']
+        self._changing[ref] = attrs = [
             attr for attr, value in msg.items()
             if not model.lookup(attr).property.matches(getattr(model, attr), value)
         ]
         try:
             model.update(**msg)
         finally:
-            del self._changing[root.ref['id']]
+            changing = [
+                attr for attr in self._changing.get(ref, [])
+                if attr not in attrs
+            ]
+            if changing:
+                self._changing[ref] = changing
+            elif ref in self._changing:
+                del self._changing[ref]
 
     def _cleanup(self, root):
         super()._cleanup(root)
@@ -253,6 +265,7 @@ class Syncable(Renderable):
             self._apply_update(events, msg, model, ref)
 
     def _process_events(self, events):
+        self._log('received events %s', events)
         busy = state.busy
         with edit_readonly(state):
             state.busy = True
@@ -260,7 +273,7 @@ class Syncable(Renderable):
         try:
             with edit_readonly(self):
                 self_events = {k: v for k, v in events.items() if '.' not in k}
-                self.param.set_param(**self_events)
+                self.param.update(**self_events)
             for k, v in self_events.items():
                 if '.' not in k:
                     continue
@@ -269,27 +282,59 @@ class Syncable(Renderable):
                 for sp in subpath:
                     obj = getattr(obj, sp)
                 with edit_readonly(obj):
-                    obj.param.set_param(**{p: v})
+                    obj.param.update(**{p: v})
+        except Exception:
+            if len(events)>1:
+                msg_end = f" changing properties {pformat(events)} \n"
+            elif len(events)==1:
+                msg_end = f" changing property {pformat(events)} \n"
+            else:
+                msg_end = "\n"
+            log.exception(f'Callback failed for object named "{self.name}"{msg_end}')
+            raise
         finally:
+            self._log('finished processing events %s', events)
             with edit_readonly(state):
                 state.busy = busy
 
-    @gen.coroutine
-    def _change_coroutine(self, doc=None):
-        self._change_event(doc)
+    def _process_bokeh_event(self, event):
+        self._log('received bokeh event %s', event)
+        busy = state.busy
+        with edit_readonly(state):
+            state.busy = True
+        try:
+            self._process_event(event)
+        finally:
+            self._log('finished processing bokeh event %s', event)
+            with edit_readonly(state):
+                state.busy = busy
+
+    async def _change_coroutine(self, doc=None):
+        if state._thread_pool:
+            state._thread_pool.submit(self._change_event, doc)
+        else:
+            with set_curdoc(doc):
+                self._change_event(doc)
+
+    async def _event_coroutine(self, event, doc):
+        if state._thread_pool:
+            state._thread_pool.submit(self._process_bokeh_event, event)
+        else:
+            with set_curdoc(doc):
+                self._process_bokeh_event(event)
 
     def _change_event(self, doc=None):
         try:
             state.curdoc = doc
-            thread = threading.current_thread()
-            thread_id = thread.ident if thread else None
-            state._thread_id = thread_id
             events = self._events
             self._events = {}
             self._process_events(events)
         finally:
             state.curdoc = None
-            state._thread_id = None
+
+    def _schedule_change(self, doc, comm):
+        with hold(doc, comm=comm):
+            self._change_event(doc)
 
     def _comm_change(self, doc, ref, comm, subpath, attr, old, new):
         if subpath:
@@ -298,8 +343,25 @@ class Syncable(Renderable):
             self._changing[ref].remove(attr)
             return
 
-        with hold(doc, comm=comm):
-            self._process_events({attr: new})
+        self._events.update({attr: new})
+        if state._thread_pool:
+            state._thread_pool.submit(self._schedule_change, doc, comm)
+        else:
+            self._schedule_change(doc, comm)
+
+    def _comm_event(self, event):
+        if state._thread_pool:
+            state._thread_pool.submit(self._process_bokeh_event, event)
+        else:
+            self._process_bokeh_event(event)
+
+    def _server_event(self, doc, event):
+        if doc.session_context and not state._unblocked(doc):
+            doc.add_next_tick_callback(
+                partial(self._event_coroutine, event, doc)
+            )
+        else:
+            self._comm_event(event)
 
     def _server_change(self, doc, ref, subpath, attr, old, new):
         if subpath:
@@ -308,17 +370,19 @@ class Syncable(Renderable):
             self._changing[ref].remove(attr)
             return
 
-        state._locks.clear()
         processing = bool(self._events)
         self._events.update({attr: new})
-        if not processing:
-            if doc.session_context:
-                doc.add_timeout_callback(
-                    partial(self._change_coroutine, doc),
-                    self._debounce
-                )
+        if processing:
+            return
+
+        if doc.session_context:
+            cb = partial(self._change_coroutine, doc)
+            if attr in self._priority_changes:
+                doc.add_next_tick_callback(cb)
             else:
-                self._change_event(doc)
+                doc.add_timeout_callback(cb, self._debounce)
+        else:
+            self._change_event(doc)
 
 
 class Reactive(Syncable, Viewable):
@@ -529,55 +593,11 @@ class Reactive(Syncable, Viewable):
         if args is None:
             args = {}
 
+        from .links import Link, assert_source_syncable, assert_target_syncable
         mapping = code or links
-        for k in mapping:
-            if k.startswith('event:'):
-                continue
-            elif hasattr(self, 'object') and isinstance(self.object, LayoutDOM):
-                current = self.object
-                for attr in k.split('.'):
-                    if not hasattr(current, attr):
-                        raise ValueError(f"Could not resolve {k} on "
-                                         f"{self.object} model. Ensure "
-                                         "you jslink an attribute that "
-                                         "exists on the bokeh model.")
-                    current = getattr(current, attr)
-            elif (k not in self.param and k not in list(self._rename.values())):
-                matches = difflib.get_close_matches(k, list(self.param))
-                if matches:
-                    matches = ' Similar parameters include: %r' % matches
-                else:
-                    matches = ''
-                raise ValueError("Could not jslink %r parameter (or property) "
-                                 "on %s object because it was not found.%s"
-                                 % (k, type(self).__name__, matches))
-            elif (self._source_transforms.get(k, False) is None or
-                  self._rename.get(k, False) is None):
-                raise ValueError("Cannot jslink %r parameter on %s object, "
-                                 "the parameter requires a live Python kernel "
-                                 "to have an effect." % (k, type(self).__name__))
-
+        assert_source_syncable(self, mapping)
         if isinstance(target, Syncable) and code is None:
-            for k, p in mapping.items():
-                if k.startswith('event:'):
-                    continue
-                elif p not in target.param and p not in list(target._rename.values()):
-                    matches = difflib.get_close_matches(p, list(target.param))
-                    if matches:
-                        matches = ' Similar parameters include: %r' % matches
-                    else:
-                        matches = ''
-                    raise ValueError("Could not jslink %r parameter (or property) "
-                                     "on %s object because it was not found.%s"
-                                    % (p, type(self).__name__, matches))
-                elif (target._source_transforms.get(p, False) is None or
-                      target._rename.get(p, False) is None):
-                    raise ValueError("Cannot jslink %r parameter on %s object "
-                                     "to %r parameter on %s object. It requires "
-                                     "a live Python kernel to have an effect."
-                                     % (k, type(self).__name__, p, type(target).__name__))
-
-        from .links import Link
+            assert_target_syncable(self, target, mapping)
         return Link(self, target, properties=links, code=code, args=args,
                     bidirectional=bidirectional)
 
@@ -642,9 +662,11 @@ class SyncableData(Reactive):
         """
         data = getattr(self, self._data_params[0])
         data[column] = array
+        if self._processed is not None:
+            self._processed[column] = array
 
     def _update_data(self, data):
-        self.param.set_param(**{self._data_params[0]: data})
+        self.param.update(**{self._data_params[0]: data})
 
     def _manual_update(self, events, model, doc, root, parent, comm):
         for event in events:
@@ -930,58 +952,88 @@ class ReactiveData(SyncableData):
     parameter between frontend and backend using a ColumnDataSource.
     """
 
+    def __init__(self, **params):
+        super().__init__(**params)
+        self._old = None
+    
     def _update_selection(self, indices):
         self.selection = indices
 
+    def _convert_column(self, values, old_values):
+        dtype = old_values.dtype
+        if dtype.kind == 'M':
+            if values.dtype.kind in 'if':
+                values = (values * 10e5).astype(dtype)
+        elif dtype.kind == 'O':
+            if (all(isinstance(ov, dt.date) for ov in old_values) and
+                not all(isinstance(iv, dt.date) for iv in values)):
+                new_values = []
+                for iv in values:
+                    if isinstance(iv, dt.datetime):
+                        iv = iv.date()
+                    elif not isinstance(iv, dt.date):
+                        iv = dt.date.fromtimestamp(iv/1000)
+                    new_values.append(iv)
+                values = new_values
+        else:
+            values = values.astype(dtype)
+        return values
+
+    def _process_data(self, data):
+        if self._updating:
+            return
+
+        # Get old data to compare to
+        old_raw, old_data = self._get_data()
+        self._old = old_raw = old_raw.copy()
+        if hasattr(old_raw, 'columns'):
+            columns = list(old_raw.columns)
+        else:
+            columns = list(old_raw)
+
+        updated = False
+        for col, values in data.items():
+            col = self._renamed_cols.get(col, col)
+            if col in self.indexes or col not in columns:
+                continue
+            if isinstance(values, dict):
+                sorted_values = sorted(values.items(), key=lambda it: int(it[0]))
+                values = [v for _, v in sorted_values]
+            values = self._convert_column(
+                np.asarray(values), old_raw[col]
+            )
+            try:
+                isequal = (old_raw[col] == values).all()
+            except Exception:
+                isequal = False
+            if not isequal:
+                self._update_column(col, values)
+                updated = True
+
+        # If no columns were updated we don't have to sync data
+        if not updated:
+            return
+
+        # Ensure we trigger events
+        self._updating = True
+        old_data = getattr(self, self._data_params[0])
+        try:
+            if old_data is self.value:
+                with param.discard_events(self):
+                    self.value = old_raw
+                self.value = old_data
+            else:
+                self.param.trigger('value')
+        finally:
+            self._updating = False
+        # Ensure that if the data was changed in a user
+        # callback, we still send the updated data
+        if old_data is not self.value:
+            self._update_cds()
+
     def _process_events(self, events):
         if 'data' in events:
-            data = events.pop('data')
-            if self._updating:
-                data = {}
-            old_raw, old_data = self._get_data()
-            if hasattr(old_raw, 'copy'):
-                old_raw = old_raw.copy()
-            elif isinstance(old_raw, dict):
-                old_raw = dict(old_raw)
-            updated = False
-            for k, v in data.items():
-                if k in self.indexes:
-                    continue
-                k = self._renamed_cols.get(k, k)
-                if isinstance(v, dict):
-                    v = [v for _, v in sorted(v.items(), key=lambda it: int(it[0]))]
-                v = np.asarray(v)
-                old_dtype = old_raw[k].dtype
-                if old_dtype.kind == 'M':
-                    v = (v * 10e5).astype(old_raw[k].dtype)
-                elif old_dtype.kind == 'O':
-                    if all(isinstance(ov, dt.date) for ov in old_raw[k]):
-                        v = np.array([dt.date.fromtimestamp(iv/1000) for iv in v])
-                else:
-                    v = v.astype(old_raw[k].dtype)
-                try:
-                    isequal = (old_data[k] == v).all()
-                except Exception:
-                    isequal = False
-                if not isequal:
-                    self._update_column(k, v)
-                    updated = True
-            if updated:
-                self._updating = True
-                old_data = getattr(self, self._data_params[0])
-                try:
-                    if old_raw is self.value:
-                        with param.discard_events(self):
-                            self.value = old_raw
-                        self.value = data
-                    else:
-                        self.param.trigger('value')
-                finally:
-                    self._updating = False
-                # Ensure that if the data was changed in a user
-                # callback, we still send the updated data
-                if old_data is not self.value:
-                    self._update_cds()
+            self._process_data(events.pop('data'))
         if 'indices' in events:
             self._updating = True
             try:
@@ -999,12 +1051,14 @@ class ReactiveHTMLMetaclass(ParameterizedMetaclass):
     HTML attributes.
     """
 
+    _loaded_extensions = set()
+
     _name_counter = Counter()
 
     _script_regex = r"script\([\"|'](.*)[\"|']\)"
 
     def __init__(mcs, name, bases, dict_):
-        from .links import PARAM_MAPPING, construct_data_model
+        from .io.datamodel import PARAM_MAPPING, construct_data_model
 
         mcs.__original_doc__ = mcs.__doc__
         ParameterizedMetaclass.__init__(mcs, name, bases, dict_)
@@ -1044,15 +1098,14 @@ class ReactiveHTMLMetaclass(ParameterizedMetaclass):
                 "ensure there is a matching </div> tag."
             )
 
-        mcs._attrs, mcs._node_callbacks = {}, {}
+        mcs._node_callbacks = {}
         mcs._inline_callbacks = []
         for node, attrs in mcs._parser.attrs.items():
             for (attr, parameters, template) in attrs:
-                param_attrs = []
                 for p in parameters:
                     if p in mcs.param or '.' in p:
-                        param_attrs.append(p)
-                    elif re.match(mcs._script_regex, p):
+                        continue
+                    if re.match(mcs._script_regex, p):
                         name = re.findall(mcs._script_regex, p)[0]
                         if name not in mcs._scripts:
                             raise ValueError(
@@ -1076,9 +1129,7 @@ class ReactiveHTMLMetaclass(ParameterizedMetaclass):
                             f"parameter or method '{p}', similar parameters "
                             f"and methods include {matches}."
                         )
-                if node not in mcs._attrs:
-                    mcs._attrs[node] = []
-                mcs._attrs[node].append((attr, param_attrs, template))
+
         ignored = list(Reactive.param)
         types = {}
         for child in mcs._parser.children.values():
@@ -1183,7 +1234,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
     Additionally we can invoke pure JS scripts defined on the class, e.g.:
 
-        <input id="input" onchange="${run_script('some_script')}"></input>
+        <input id="input" onchange="${script('some_script')}"></input>
 
     This will invoke the following script if it is defined on the class:
 
@@ -1230,6 +1281,8 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
     _dom_events = {}
 
+    _extension_name = None
+
     _template = ""
 
     _scripts = {}
@@ -1238,6 +1291,10 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
     __abstract = True
 
+    __css__ = None
+    __javascript__ = None
+    __javascript_modules__ = None
+
     def __init__(self, **params):
         from .pane import panel
         for children_param in self._parser.children.values():
@@ -1262,9 +1319,20 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             else:
                 params[children_param] = panel(child_value)
         super().__init__(**params)
+        self._attrs = {}
         self._panes = {}
         self._event_callbacks = defaultdict(lambda: defaultdict(list))
 
+    @classmethod
+    def _loaded(cls):
+        """
+        Whether the component has been loaded.
+        """
+        return (
+            cls._extension_name is None or
+            cls._extension_name in ReactiveHTMLMetaclass._loaded_extensions
+        )
+
     def _cleanup(self, root):
         for child, panes in self._panes.items():
             for pane in panes:
@@ -1292,7 +1360,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             if getattr(self, p) is not None and p != 'name'
         }
         data_params = {}
-        for k, v in self.param.get_param_values():
+        for k, v in self.param.values().items():
             if (
                 (k in ignored and k != 'name') or
                 ((self.param[k].precedence or 0) < 0) or
@@ -1302,13 +1370,14 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             if isinstance(v, str):
                 v = bleach.clean(v)
             data_params[k] = v
+        html, nodes, self._attrs = self._get_template()
         params.update({
             'attrs': self._attrs,
             'callbacks': self._node_callbacks,
             'data': self._data_model(**self._process_param_change(data_params)),
             'events': self._get_events(),
-            'html': escape(textwrap.dedent(self._get_template())),
-            'nodes': self._parser.nodes,
+            'html': escape(textwrap.dedent(html)),
+            'nodes': nodes,
             'looped': [node for node, _ in self._parser.looped],
             'scripts': {}
         })
@@ -1420,7 +1489,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         # Render Jinja template
         template = jinja2.Template(template_string)
         context = {'param': self.param, '__doc__': self.__original_doc__, 'id': id}
-        for parameter, value in self.param.get_param_values():
+        for parameter, value in self.param.values().items():
             context[parameter] = value
             if parameter in self._child_names:
                 context[f'{parameter}_names'] = self._child_names[parameter]
@@ -1445,6 +1514,18 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 .replace(f'id="{name}"', f'id="{name}-${{id}}"')
             )
 
+        # Parse attrs
+        p_attrs = {}
+        for node, attrs in parser.attrs.items():
+            for (attr, parameters, template) in attrs:
+                param_attrs = []
+                for p in parameters:
+                    if p in self.param or '.' in p:
+                        param_attrs.append(p)
+                if node not in p_attrs:
+                    p_attrs[node] = []
+                p_attrs[node].append((attr, param_attrs, template))
+
         # Remove child node template syntax
         for parent, child_name in self._parser.children.items():
             if (parent, child_name) in self._parser.looped:
@@ -1452,7 +1533,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                     html = html.replace('${%s[%d]}' % (child_name, i), '')
             else:
                 html = html.replace('${%s}' % child_name, '')
-        return html
+        return html, parser.nodes, p_attrs
 
     def _linked_properties(self):
         linked_properties = [p for pss in self._attrs.values() for _, ps, _ in pss for p in ps]
@@ -1471,13 +1552,35 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
     def _get_model(self, doc, root=None, parent=None, comm=None):
         properties = self._process_param_change(self._init_params())
         model = _BkReactiveHTML(**properties)
+        if comm and not self._loaded():
+            self.param.warning(
+                f'{type(self).__name__} was not imported on instantiation and may not '
+                'render in a notebook. Restart the notebook kernel and '
+                'ensure you load it as part of the extension using:'
+                f'\n\npn.extension(\'{self._extension_name}\')\n'
+            )
+        elif root is not None and not self._loaded() and root.ref['id'] in state._views:
+            self.param.warning(
+                f'{type(self).__name__} was not imported on instantiation may not '
+                'render in the served application. Ensure you add the '
+                'following to the top of your application:'
+                f'\n\npn.extension(\'{self._extension_name}\')\n'
+            )
+        if self._extension_name:
+            ReactiveHTMLMetaclass._loaded_extensions.add(self._extension_name)
+
         if not root:
             root = model
+
         for p, v in model.data.properties_with_values().items():
             if isinstance(v, DataModel):
                 v.tags.append(f"__ref:{root.ref['id']}")
         model.children = self._get_children(doc, root, model, comm)
-        model.on_event('dom_event', self._process_event)
+
+        if comm:
+            model.on_event('dom_event', self._comm_event)
+        else:
+            model.on_event('dom_event', partial(self._server_event, doc))
 
         self._link_props(model.data, self._linked_properties(), doc, root, comm)
         self._models[root.ref['id']] = (model, parent)
@@ -1541,7 +1644,10 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 data_msg[prop] = v
         if new_children:
             if self._parser.looped:
-                model_msg['html'] = escape(self._get_template())
+                html, nodes, self._attrs = self._get_template()
+                model_msg['attrs'] = self._attrs
+                model_msg['nodes'] = nodes
+                model_msg['html'] = escape(textwrap.dedent(html))
             children = self._get_children(doc, root, model, comm)
         else:
             children = None
