@@ -6,6 +6,7 @@ import pathlib
 import param
 
 from ...depends import depends
+from ...io.resources import CSS_URLS
 from ...layout import Card, GridSpec
 from ..base import BasicTemplate
 from ..theme import DarkTheme, DefaultTheme
@@ -56,8 +57,8 @@ class ReactTemplate(BasicTemplate):
             'react-grid': "https://cdnjs.cloudflare.com/ajax/libs/react-grid-layout/1.1.1/react-grid-layout.min.js"
         },
         'css': {
-            'bootstrap': "https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css",
-            'font-awesome': "https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css"
+            'bootstrap': CSS_URLS['bootstrap4'],
+            'font-awesome': CSS_URLS['font-awesome']
         }
     }
 
