@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals, print_function
-
 import os
 import json
 import glob
@@ -109,12 +107,17 @@ def test_embed_float_slider_explicit_values(document, comm):
         content = json.loads(v['content'])
         assert 'events' in content
         events = content['events']
-        assert len(events) == 1
-        event = events[0]
-        assert event['kind'] == 'ModelChanged'
-        assert event['attr'] == 'text'
-        assert event['model'] == model.children[1].ref
-        assert event['new'] == '&lt;pre&gt;%s&lt;/pre&gt;' % states[k]
+        assert len(events) == 2
+        event1, event2 = events
+        assert event1['kind'] == 'ModelChanged'
+        assert event1['attr'] == 'text'
+        assert event1['model'] == model.children[0].children[0].ref
+        assert event1['new'] == '<b>%s</b>' % states[k]
+
+        assert event2['kind'] == 'ModelChanged'
+        assert event2['attr'] == 'text'
+        assert event2['model'] == model.children[1].ref
+        assert event2['new'] == '&lt;pre&gt;%s&lt;/pre&gt;' % states[k]
 
 
 def test_embed_select_explicit_values(document, comm):
@@ -367,12 +370,17 @@ def test_embed_slider_str_link(document, comm):
         content = json.loads(v['content'])
         assert 'events' in content
         events = content['events']
-        assert len(events) == 1
-        event = events[0]
-        assert event['kind'] == 'ModelChanged'
-        assert event['attr'] == 'text'
-        assert event['model'] == model.children[1].ref
-        assert event['new'] == '&lt;pre&gt;%.1f&lt;/pre&gt;' % values[k]
+        assert len(events) == 2
+        event1, event2 = events
+        assert event1['kind'] == 'ModelChanged'
+        assert event1['attr'] == 'text'
+        assert event1['model'] == model.children[0].children[0].ref
+        assert event1['new'] == '<b>%d</b>' % values[k]
+
+        assert event2['kind'] == 'ModelChanged'
+        assert event2['attr'] == 'text'
+        assert event2['model'] == model.children[1].ref
+        assert event2['new'] == '&lt;pre&gt;%.1f&lt;/pre&gt;' % values[k]
 
 
 def test_embed_slider_str_jslink(document, comm):
@@ -448,20 +456,29 @@ def test_embed_merged_sliders(document, comm):
     assert len(cbs) == 5
 
     ref1, ref2 = model.children[2].ref['id'], model.children[3].ref['id']
+    print(model.children)
+    ref3 = model.children[0].children[0].ref['id']
+    ref4 = model.children[1].children[0].ref['id']
     state0 = json.loads(state_model.state[0]['content'])['events']
     assert state0 == [
-        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref1}, "new": "1"},
-        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref2}, "new": "1"}
+        {'attr': 'text', 'kind': 'ModelChanged', 'model': {'id': ref3}, 'new': 'A: <b>1</b>', 'hint': None},
+        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref1}, "new": "1", "hint": None},
+        {'attr': 'text', 'kind': 'ModelChanged', 'model': {'id': ref4}, 'new': 'A: <b>1</b>', 'hint': None},
+        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref2}, "new": "1", "hint": None}
     ]
     state1 = json.loads(state_model.state[1]['content'])['events']
     assert state1 == [
-        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref1}, "new": "5"},
-        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref2}, "new": "5"}
+        {'attr': 'text', 'kind': 'ModelChanged', 'model': {'id': ref3}, 'new': 'A: <b>5</b>', 'hint': None},
+        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref1}, "new": "5", "hint": None},
+        {'attr': 'text', 'kind': 'ModelChanged', 'model': {'id': ref4}, 'new': 'A: <b>5</b>', 'hint': None},
+        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref2}, "new": "5", "hint": None}
     ]
     state2 = json.loads(state_model.state[2]['content'])['events']
     assert state2 == [
-        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref1}, "new": "9"},
-        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref2}, "new": "9"}
+        {'attr': 'text', 'kind': 'ModelChanged', 'model': {'id': ref3}, 'new': 'A: <b>9</b>', 'hint': None},
+        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref1}, "new": "9", "hint": None},
+        {'attr': 'text', 'kind': 'ModelChanged', 'model': {'id': ref4}, 'new': 'A: <b>9</b>', 'hint': None},
+        {"attr": "text", "kind": "ModelChanged", "model": {"id": ref2}, "new": "9", "hint": None}
     ]
 
 
