@@ -8,15 +8,15 @@ from panel.layout import (
     Accordion, Card, Column, Row, Tabs, Spacer, WidgetBox
 )
 from panel.layout.base import ListPanel, NamedListPanel
+from panel.widgets import Debugger
 from panel.param import Param
 from panel.pane import Bokeh
-from panel.tests.util import check_layoutable_properties, py3_only
-
+from panel.tests.util import check_layoutable_properties
 
+excluded = (NamedListPanel, Debugger)
 all_panels = [w for w in param.concrete_descendents(ListPanel).values()
-               if not w.__name__.startswith('_') and w is not NamedListPanel]
+               if not w.__name__.startswith('_') and not issubclass(w, excluded)]
 
-@py3_only
 @pytest.mark.parametrize('panel', all_panels)
 def test_layout_signature(panel):
     from inspect import signature
@@ -469,7 +469,7 @@ def test_layout_with_param_setitem(document, comm):
         select = param.ObjectSelector(default=0, objects=[0,1])
 
         def __init__(self, **params):
-            super(TestClass, self).__init__(**params)
+            super().__init__(**params)
             self._layout = Row(Param(self.param, parameters=['select']),
                                self.select)
 
