@@ -2,13 +2,22 @@
 # pylint: disable=missing-function-docstring,missing-module-docstring,missing-class-docstring
 import holoviews as hv
 import numpy as np
-import panel as pn
+
+from bokeh.document import Document
 from holoviews import opts
 
+from panel.config import config
+from panel.layout import Column
+from panel.pane import HoloViews, HTML, Markdown
+from panel.param import Param
 from panel.template.fast.grid import FastGridTemplate, FastGridDarkTheme
+from panel.widgets import Button
 
 hv.extension("bokeh")
-opts.defaults(opts.Ellipse(line_width=3, color="#DF3874"))
+ACCENT_COLOR = "#f63366" # "lightblue"
+
+opts.defaults(opts.Ellipse(line_width=3, color=ACCENT_COLOR))
+
 
 
 
@@ -16,7 +25,7 @@ def test_template_theme_parameter():
     template = FastGridTemplate(title="Fast", theme="dark")
     # Not '#3f3f3f' which is for the Vanilla theme
 
-    doc = template.server_doc()
+    doc = template.server_doc(Document())
     assert doc.theme._json['attrs']['Figure']['background_fill_color']=="#181818"
 
     assert isinstance(template._get_theme(), FastGridDarkTheme)
@@ -24,7 +33,7 @@ def test_template_theme_parameter():
 
 
 COLLAPSED_ICON = """
-<svg style="stroke: #E62F63" width="18" height="18" viewBox="0 0 18 18" fill="none" xmlns="http://www.w3.org/2000/svg" slot="collapsed-icon">
+<svg style="stroke: var(--accent-fill-rest);" width="18" height="18" viewBox="0 0 18 18" fill="none" xmlns="http://www.w3.org/2000/svg" slot="collapsed-icon">
             <path d="M15.2222 1H2.77778C1.79594 1 1 1.79594 1 2.77778V15.2222C1 16.2041 1.79594 17 2.77778 17H15.2222C16.2041 17 17 16.2041 17 15.2222V2.77778C17 1.79594 16.2041 1 15.2222 1Z" stroke-linecap="round" stroke-linejoin="round"></path>
             <path d="M9 5.44446V12.5556" stroke-linecap="round" stroke-linejoin="round"></path>
             <path d="M5.44446 9H12.5556" stroke-linecap="round" stroke-linejoin="round"></path>
@@ -32,7 +41,7 @@ COLLAPSED_ICON = """
 """ # noqa
 
 EXPANDED_ICON = """
-<svg style="stroke: #E62F63" width="18" height="18" viewBox="0 0 18 18" fill="none" xmlns="http://www.w3.org/2000/svg" slot="expanded-icon">
+<svg style="stroke: var(--accent-fill-rest);" width="18" height="18" viewBox="0 0 18 18" fill="none" xmlns="http://www.w3.org/2000/svg" slot="expanded-icon">
     <path d="M15.2222 1H2.77778C1.79594 1 1 1.79594 1 2.77778V15.2222C1 16.2041 1.79594 17 2.77778 17H15.2222C16.2041 17 17 16.2041 17 15.2222V2.77778C17 1.79594 16.2041 1 15.2222 1Z" stroke-linecap="round" stroke-linejoin="round"></path>
     <path d="M5.44446 9H12.5556" stroke-linecap="round" stroke-linejoin="round"></path>
 </svg>
@@ -104,18 +113,18 @@ def _create_hvplot():
 
 
 def _navigation_menu():
-    return pn.pane.HTML(NAVIGATION_HTML)
+    return HTML(NAVIGATION_HTML)
 
 
 def _sidebar_items():
     return [
-        pn.pane.Markdown("## Settings"),
+        Markdown("## Settings"),
         _navigation_menu(),
     ]
 
 
 def _fast_button_card():
-    button = pn.widgets.Button(name="Click me", button_type="primary")
+    button = Button(name="Click me", button_type="primary")
     button.param.name.precedence = 0
     button.param.clicks.precedence = 0
     button.param.disabled.precedence = 0
@@ -129,34 +138,43 @@ def _fast_button_card():
         "height",
         "sizing_mode",
     ]
-    settings = pn.Param(
+    settings = Param(
         button,
         parameters=button_parameters,
         show_name=False,
         sizing_mode="stretch_width",
     )
-    return pn.Column(
-        pn.pane.HTML("<h2>Button</h2>"),
+    return Column(
+        HTML("<h2>Button</h2>"),
         button,
-        pn.pane.HTML("<h3>Parameters</h3>"),
+        HTML("<h3>Parameters</h3>"),
         settings,
         sizing_mode="stretch_both",
     )
 
 
 def test_app():
-    pn.config.sizing_mode = "stretch_width"
     app = FastGridTemplate(
-        title="FastGridTemplate",
+        title="FastGridTemplate w. Layout Persistence",
+        site="Panel",
+        accent_base_color=ACCENT_COLOR,
+        header_background=ACCENT_COLOR,
+        # header_color="#000000",
+        header_accent_base_color="#FFFFFF",
+        row_height=50,
+        prevent_collision=True,
+        # main_layout="",
+        save_layout=True,
     )
-    app.main[0:6, 0:6] = pn.pane.Markdown(INFO, sizing_mode="stretch_both")
-    app.main[0:6, 6:12] = pn.pane.HoloViews(_create_hvplot(), sizing_mode="stretch_both")
-    app.main[6:18, 0:3] = _fast_button_card()
-    app.main[6:12, 6:12] = pn.pane.HoloViews(_create_hvplot(), sizing_mode="stretch_both")
+    app.main[0:7, 0:6] = Markdown(INFO, sizing_mode="stretch_both")
+    app.main[0:7, 6:12] = HoloViews(_create_hvplot(), sizing_mode="stretch_both")
+    app.main[7:18, 0:6] = _fast_button_card()
+    app.main[7:14, 6:12] = HoloViews(_create_hvplot(), sizing_mode="stretch_both")
     app.sidebar.extend(_sidebar_items())
 
     return app
 
 
 if __name__.startswith("bokeh"):
+    config.sizing_mode = "stretch_width"
     test_app().servable()
