@@ -1,23 +1,26 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import param
 import pytest
 
 from panel.io import block_comm
+from panel.layout import Row
+from panel.links import CallbackGenerator
 from panel.widgets import (
-    CompositeWidget, DataFrame, Dial, FileDownload, FloatSlider,
-    TextInput, ToggleGroup, Widget
+    CompositeWidget, Dial, FileDownload, FloatSlider, LinearGauge,
+    TextInput, Terminal, ToggleGroup, Tqdm, Widget
+)
+from panel.widgets.tables import BaseTable
+from panel.tests.util import check_layoutable_properties
+
+excluded = (
+    BaseTable, CompositeWidget, Dial, FileDownload, LinearGauge,
+    ToggleGroup, Terminal, Tqdm 
 )
-from panel.tests.util import check_layoutable_properties, py3_only
 
 all_widgets = [
     w for w in param.concrete_descendents(Widget).values()
-    if not w.__name__.startswith('_') and
-    not issubclass(w, (CompositeWidget, DataFrame, FileDownload, ToggleGroup, Dial))
+    if not w.__name__.startswith('_') and not issubclass(w, excluded)
 ]
 
-
-@py3_only
 @pytest.mark.parametrize('widget', all_widgets)
 def test_widget_signature(widget):
     from inspect import signature
@@ -25,6 +28,19 @@ def test_widget_signature(widget):
     assert len(parameters) == 1
 
 
+@pytest.mark.parametrize('widget', all_widgets)
+def test_widget_linkable_params(widget):
+    w = widget()
+    controls = w.controls(jslink=True)
+    layout = Row(w, controls)
+
+    try:
+        CallbackGenerator.error = True
+        layout.get_root()
+    finally:
+        CallbackGenerator.error = False
+
+
 @pytest.mark.parametrize('widget', all_widgets)
 def test_widget_layout_properties(widget, document, comm):
     w = widget()
@@ -48,8 +64,8 @@ def test_widget_clone(widget):
     w = widget()
     clone = w.clone()
 
-    assert ([(k, v) for k, v in sorted(w.param.get_param_values()) if k != 'name'] ==
-            [(k, v) for k, v in sorted(clone.param.get_param_values()) if k != 'name'])
+    assert ([(k, v) for k, v in sorted(w.param.values().items()) if k != 'name'] ==
+            [(k, v) for k, v in sorted(clone.param.values().items()) if k != 'name'])
 
 
 @pytest.mark.parametrize('widget', all_widgets)
@@ -57,8 +73,8 @@ def test_widget_clone_override(widget):
     w = widget()
     clone = w.clone(width=50)
 
-    assert ([(k, v) for k, v in sorted(w.param.get_param_values()) if k not in ['name', 'width']] ==
-            [(k, v) for k, v in sorted(clone.param.get_param_values()) if k not in ['name', 'width']])
+    assert ([(k, v) for k, v in sorted(w.param.values().items()) if k not in ['name', 'width']] ==
+            [(k, v) for k, v in sorted(clone.param.values().items()) if k not in ['name', 'width']])
     assert clone.width == 50
     assert w.width is widget.width
 
@@ -87,14 +103,14 @@ def test_widget_triggers_events(document, comm):
     document.hold()
 
     # Simulate client side change
-    document._held_events = document._held_events[:-1]
+    document.callbacks._held_events = document.callbacks._held_events[:-1]
 
     # Set new value
     with block_comm():
         text.value = '123'
 
-    assert len(document._held_events) == 1
-    event = document._held_events[0]
+    assert len(document.callbacks._held_events) == 1
+    event = document.callbacks._held_events[0]
     assert event.attr == 'value'
     assert event.model is widget
     assert event.new == '123'
@@ -116,6 +132,22 @@ def test_widget_from_param_cls():
     assert Test.a == 'def'
 
 
+def test_widget_from_param_negative_precedence():
+    class Test(param.Parameterized):
+
+        a = param.Parameter(precedence=-1)
+
+    widget = TextInput.from_param(Test.param.a)
+    assert isinstance(widget, TextInput)
+    assert widget.name == 'A'
+
+    Test.a = 'abc'
+    assert widget.value == 'abc'
+
+    widget.value = 'def'
+    assert Test.a == 'def'
+
+
 def test_widget_from_param_instance():
     class Test(param.Parameterized):
 
@@ -151,3 +183,4 @@ def test_widget_from_param_instance_with_kwargs():
 
     widget.value = 4.3
     assert test.a == 4.3
+
