@@ -3,6 +3,8 @@ Defines the Widget base class which provides bi-directional
 communication between the rendered dashboard and the Widget
 parameters.
 """
+import math
+
 import param
 
 from ..layout import Row
@@ -69,7 +71,10 @@ class Widget(Reactive):
         Widget instance linked to the supplied parameter
         """
         from ..param import Param
-        layout = Param(parameter, widgets={parameter.name: dict(type=cls, **params)})
+        layout = Param(
+            parameter, widgets={parameter.name: dict(type=cls, **params)},
+            display_threshold=-math.inf
+        )
         return layout[0]
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
@@ -77,7 +82,7 @@ class Widget(Reactive):
         if root is None:
             root = model
         # Link parameters and bokeh model
-        values = dict(self.param.get_param_values())
+        values = self.param.values()
         properties = self._filter_properties(list(self._process_param_change(values)))
         self._models[root.ref['id']] = (model, parent)
         self._link_props(model, properties, doc, root, comm)
@@ -141,7 +146,7 @@ class CompositeWidget(Widget):
 
     def _update_layout_params(self, *events):
         updates = {event.name: event.new for event in events}
-        self._composite.param.set_param(**updates)
+        self._composite.param.update(**updates)
 
     def select(self, selector=None):
         """
