@@ -1,6 +1,6 @@
 """
-Defines Button and button-like widgets which allow triggering events
-or merely toggling between on-off states.
+Defines the Button and button-like widgets which allow triggering
+events or merely toggling between on-off states.
 """
 from functools import partial
 
@@ -10,7 +10,7 @@ from bokeh.models import (
     Button as _BkButton, Toggle as _BkToggle, Dropdown as _BkDropdown
 )
 
-from bokeh.events import (MenuItemClick, ButtonClick)
+from bokeh.events import MenuItemClick, ButtonClick
 
 from .base import Widget
 
@@ -19,7 +19,10 @@ BUTTON_TYPES = ['default', 'primary', 'success', 'warning', 'danger','light']
 
 class _ButtonBase(Widget):
 
-    button_type = param.ObjectSelector(default='default', objects=BUTTON_TYPES)
+    button_type = param.ObjectSelector(default='default', objects=BUTTON_TYPES, doc="""
+        A button theme; should be one of 'default' (white), 'primary'
+        (blue), 'success' (green), 'info' (yellow), 'light' (light),
+        or 'danger' (red).""")
 
     _rename = {'name': 'label'}
 
@@ -34,8 +37,10 @@ class _ClickButton(_ButtonBase):
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
         model = super()._get_model(doc, root, parent, comm)
-        ref = (root or model).ref['id']
-        model.on_click(partial(self._server_click, doc, ref))
+        if comm:
+            model.on_event(self._event, self._comm_event)
+        else:
+            model.on_event(self._event, partial(self._server_event, doc))
         return model
 
     def js_on_click(self, args={}, code=""):
@@ -87,10 +92,28 @@ class _ClickButton(_ButtonBase):
 
 
 class Button(_ClickButton):
+    """
+    The `Button` widget allows triggering events when the button is
+    clicked.
 
-    clicks = param.Integer(default=0)
+    The Button provides a `value` parameter, which will toggle from
+    `False` to `True` while the click event is being processed
 
-    value = param.Event()
+    It also provides an additional `clicks` parameter, that can be
+    watched to subscribe to click events.
+
+    Reference: https://panel.holoviz.org/reference/widgets/Button.html#widgets-gallery-button
+
+    :Example:
+
+    >>> pn.widgets.Button(name='Click me', button_type='primary')
+    """
+
+    clicks = param.Integer(default=0, doc="""
+        Number of clicks (can be listened to)""")
+
+    value = param.Event(doc="""
+        Toggles from False to True while the event is being processed.""")
 
     _rename = {'clicks': None, 'name': 'label', 'value': None}
 
@@ -108,27 +131,35 @@ class Button(_ClickButton):
 
     jslink.__doc__ = Widget.jslink.__doc__
 
-    def _server_click(self, doc, ref, event):
-        processing = bool(self._events)
-        self._events.update({"clicks": self.clicks+1})
+    def _process_event(self, event):
         self.param.trigger('value')
-        if not processing:
-            if doc.session_context:
-                doc.add_timeout_callback(partial(self._change_coroutine, doc), self._debounce)
-            else:
-                self._change_event(doc)
-
-    def _process_property_change(self, msg):
-        msg = super()._process_property_change(msg)
-        if 'clicks' in msg:
-            msg['clicks'] = self.clicks + 1
-        return msg
+        self.clicks += 1
 
     def on_click(self, callback):
+        """
+        Register a callback to be executed when the `Button` is clicked.
+
+        The callback is given an `Event` argument declaring the number of clicks
+
+        Arguments
+        ---------
+        callback: (callable)
+            The function to run on click events. Must accept a positional `Event` argument
+        """
         self.param.watch(callback, 'clicks', onlychanged=False)
 
 
 class Toggle(_ButtonBase):
+    """The `Toggle` widget allows toggling a single condition between `True`/`False` states.
+
+    This widget is interchangeable with the `Checkbox` widget.
+
+    Reference: https://panel.holoviz.org/reference/widgets/Toggle.html
+
+    :Example:
+
+    >>> Toggle(name='Toggle', button_type='success')
+    """
 
     value = param.Boolean(default=False, doc="""
         Whether the button is currently toggled.""")
@@ -145,6 +176,22 @@ class Toggle(_ButtonBase):
 
 
 class MenuButton(_ClickButton):
+    """
+    The `MenuButton` widget allows specifying a list of menu items to
+    select from triggering events when the button is clicked.
+
+    Unlike other widgets, it does not have a `value`
+    parameter. Instead it has a `clicked` parameter that can be
+    watched to trigger events and which reports the last clicked menu
+    item.
+
+    Reference: https://panel.holoviz.org/reference/widgets/MenuButton.html
+
+    :Example:
+
+    >>> menu_items = [('Option A', 'a'), ('Option B', 'b'), None, ('Option C', 'c')]
+    >>> MenuButton(name='Dropdown', items=menu_items, button_type='primary')
+    """
 
     clicked = param.String(default=None, doc="""
       Last menu item that was clicked.""")
@@ -162,17 +209,22 @@ class MenuButton(_ClickButton):
 
     _event = 'menu_item_click'
 
-    def on_click(self, callback):
-        self.param.watch(callback, 'clicked', onlychanged=False)
-
-    def _server_click(self, doc, ref, event):
-        processing = bool(self._events)
+    def _process_event(self, event):
         if isinstance(event, MenuItemClick):
-            self._events.update({"clicked": event.item})
+            item = event.item
         elif isinstance(event, ButtonClick):
-            self._events.update({"clicked": self.name})
-        if not processing:
-            if doc.session_context:
-                doc.add_timeout_callback(partial(self._change_coroutine, doc), self._debounce)
-            else:
-                self._change_event(doc)
+            item = self.name
+        self.clicked = item
+
+    def on_click(self, callback):
+        """
+        Register a callback to be executed when the button is clicked.
+
+        The callback is given an `Event` argument declaring the number of clicks
+
+        Arguments
+        ---------
+        callback: (callable)
+            The function to run on click events. Must accept a positional `Event` argument
+        """
+        self.param.watch(callback, 'clicked', onlychanged=False)
