@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Defines a FileSelector widget which allows selecting files and
 directories on the server.
@@ -13,6 +12,7 @@ import param
 from ..io import PeriodicCallback
 from ..layout import Column, Divider, Row
 from ..viewable import Layoutable
+from ..util import fullpath
 from .base import CompositeWidget
 from .button import Button
 from .input import TextInput
@@ -54,6 +54,16 @@ def scan_path(path, file_pattern='*'):
 
 
 class FileSelector(CompositeWidget):
+    """
+    The `FileSelector` widget allows browsing the filesystem on the
+    server and selecting one or more files in a directory.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/FileSelector.html
+
+    :Example:
+
+    >>> FileSelector(directory='~', file_pattern='*.png')
+    """
 
     directory = param.String(default=os.getcwd(), doc="""
         The directory to explore.""")
@@ -93,10 +103,10 @@ class FileSelector(CompositeWidget):
     def __init__(self, directory=None, **params):
         from ..pane import Markdown
         if directory is not None:
-            params['directory'] = os.path.abspath(os.path.expanduser(directory))
+            params['directory'] = fullpath(directory)
         if 'root_directory' in params:
             root = params['root_directory']
-            params['root_directory'] = os.path.abspath(os.path.expanduser(root))
+            params['root_directory'] = fullpath(root)
         if params.get('width') and params.get('height') and 'sizing_mode' not in params:
             params['sizing_mode'] = None
 
@@ -163,7 +173,7 @@ class FileSelector(CompositeWidget):
         self.value = value
 
     def _dir_change(self, event):
-        path = os.path.abspath(os.path.expanduser(self._directory.value))
+        path = fullpath(self._directory.value)
         if not path.startswith(self._root_directory):
             self._directory.value = self._root_directory
             return
@@ -175,7 +185,7 @@ class FileSelector(CompositeWidget):
         self._update_files(refresh=True)
 
     def _update_files(self, event=None, refresh=False):
-        path = os.path.abspath(self._directory.value)
+        path = fullpath(self._directory.value)
         refresh = refresh or (event and getattr(event, 'obj', None) is self._reload)
         if refresh:
             path = self._cwd
@@ -234,7 +244,7 @@ class FileSelector(CompositeWidget):
             return
 
         relpath = event.new[0].replace('📁', '')
-        sel = os.path.abspath(os.path.join(self._cwd, relpath))
+        sel = fullpath(os.path.join(self._cwd, relpath))
         if os.path.isdir(sel):
             self._directory.value = sel
         else:
