@@ -7,8 +7,8 @@ import json
 
 from base64 import b64decode
 from datetime import datetime, date
-from six import string_types
 
+import numpy as np
 import param
 
 from bokeh.models.formatters import TickFormatter
@@ -17,16 +17,26 @@ from bokeh.models.widgets import (
     DatePicker as _BkDatePicker, Div as _BkDiv, TextInput as _BkTextInput,
     PasswordInput as _BkPasswordInput, Spinner as _BkSpinner,
     FileInput as _BkFileInput, TextAreaInput as _BkTextAreaInput,
-    NumericInput as _BkNumericInput)
+    NumericInput as _BkNumericInput
+)
 
 from ..config import config
 from ..layout import Column
-from ..util import param_reprs, as_unicode
+from ..util import param_reprs
 from .base import Widget, CompositeWidget
 from ..models import DatetimePicker as _bkDatetimePicker
 
 
 class TextInput(Widget):
+    """
+    The `TextInput` widget allows entering any string using a text input box.
+
+    Reference: https://panel.holoviz.org/reference/widgets/TextInput.html
+
+    :Example:
+
+    >>> TextInput(name='Name', placeholder='Enter your name here ...')
+    """
 
     max_length = param.Integer(default=5000, doc="""
       Max count of characters in the input field.""")
@@ -44,16 +54,58 @@ class TextInput(Widget):
 
 
 class PasswordInput(TextInput):
+    """
+    The `PasswordInput` allows entering any string using an obfuscated text
+    input box.
+
+    Reference: https://panel.holoviz.org/reference/widgets/PasswordInput.html
+
+    :Example:
+
+    >>> PasswordInput(
+    ...     name='Password', placeholder='Enter your password here...'
+    ... )
+    """
 
     _widget_type = _BkPasswordInput
 
 
 class TextAreaInput(TextInput):
+    """
+    The `TextAreaInput` allows entering any multiline string using a text input
+    box.
+    
+    Lines are joined with the newline character `\n`.
+
+    Reference: https://panel.holoviz.org/reference/widgets/TextAreaInput.html
+
+    :Example:
+
+    >>> TextAreaInput(
+    ...     name='Description', placeholder='Enter your description here...'
+    ... )
+    """
 
     _widget_type = _BkTextAreaInput
 
 
 class FileInput(Widget):
+    """
+    The `FileInput` allows the user to upload one or more files to the server.
+    
+    It makes the filename, MIME type and (bytes) content available in Python.
+    
+    Please note 
+    
+    - you can in fact *drag and drop* files onto the `FileInput`.
+    - you easily save the files using the `save` method.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/FileInput.html
+
+    :Example:
+
+    >>> FileInput(accept='.png,.jpeg', multiple=True)
+    """
 
     accept = param.String(default=None)
 
@@ -88,7 +140,7 @@ class FileInput(Widget):
     def _process_property_change(self, msg):
         msg = super()._process_property_change(msg)
         if 'value' in msg:
-            if isinstance(msg['value'], string_types):
+            if isinstance(msg['value'], str):
                 msg['value'] = b64decode(msg['value'])
             else:
                 msg['value'] = [b64decode(content) for content in msg['value']]
@@ -96,25 +148,56 @@ class FileInput(Widget):
 
     def save(self, filename):
         """
-        Saves the uploaded FileInput data to a file or BytesIO object.
+        Saves the uploaded FileInput data object(s) to file(s) or
+        BytesIO object(s).
 
         Arguments
         ---------
-        filename (str): File path or file-like object
+        filename (str or list[str]): File path or file-like object
         """
-        if isinstance(filename, string_types):
-            with open(filename, 'wb') as f:
-                f.write(self.value)
-        else:
-            filename.write(self.value)
+        value = self.value
+        if isinstance(filename, list) and not isinstance(value, list):
+            raise TypeError(
+                "FileInput contains a list of files but only a single "
+                "filename was given. Please provide a list of filenames or "
+                "file-like objects."
+            )
+        elif not isinstance(filename, list) and isinstance(value, list):
+            raise TypeError(
+                "FileInput contains a single files but a list of "
+                "filenames was given. Please provide a single filename "
+                "or file-like object."
+            )
+        elif not isinstance(value, list):
+            value = [self.value]
+        elif not isinstance(filename, list):
+            filename = [filename]
+
+        for val, fn in zip(value, filename):
+            if isinstance(fn, str):
+                with open(fn, 'wb') as f:
+                    f.write(val)
+            else:
+                fn.write(val)
 
 
 class StaticText(Widget):
+    """
+    The `StaticText` widget displays a text value, but does not allow editing
+    it.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/StaticText.html
+
+    :Example:
+
+    >>> StaticText(name='Model', value='animagen2')
+    """
 
     style = param.Dict(default=None, doc="""
         Dictionary of CSS property:value pairs to apply to this Div.""")
 
-    value = param.Parameter(default=None)
+    value = param.Parameter(default=None, doc="""
+        The current value""")
 
     _format = '<b>{title}</b>: {value}'
 
@@ -129,7 +212,7 @@ class StaticText(Widget):
     def _process_param_change(self, msg):
         msg = super()._process_property_change(msg)
         if 'value' in msg:
-            text = as_unicode(msg.pop('value'))
+            text = str(msg.pop('value'))
             partial = self._format.replace('{value}', '').format(title=self.name)
             if self.name:
                 text = self._format.format(title=self.name, value=text.replace(partial, ''))
@@ -138,12 +221,29 @@ class StaticText(Widget):
 
 
 class DatePicker(Widget):
+    """
+    The `DatePicker` allows selecting selecting a `date` value using a text box
+    and a date-picking utility.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/DatePicker.html
 
-    value = param.CalendarDate(default=None)
+    :Example:
 
-    start = param.CalendarDate(default=None)
+    >>> DatePicker(
+    ...     value=date(2025,1,1),
+    ...     start=date(2025,1,1), end=date(2025,12,31),
+    ...     name='Date'
+    ... )
+    """
 
-    end = param.CalendarDate(default=None)
+    value = param.CalendarDate(default=None, doc="""
+        The current value""")
+
+    start = param.CalendarDate(default=None, doc="""
+        Inclusive lower bound of the allowed date selection""")
+
+    end = param.CalendarDate(default=None, doc="""
+        Inclusive upper bound of the allowed date selection""")
 
     disabled_dates = param.List(default=None, class_=(date, str))
 
@@ -158,26 +258,37 @@ class DatePicker(Widget):
     def _process_property_change(self, msg):
         msg = super()._process_property_change(msg)
         if 'value' in msg:
-            if isinstance(msg['value'], string_types):
+            if isinstance(msg['value'], str):
                 msg['value'] = datetime.date(datetime.strptime(msg['value'], '%Y-%m-%d'))
         return msg
 
 
 class _DatetimePickerBase(Widget):
 
-    start = param.CalendarDate(default=None)
+    disabled_dates = param.List(default=None, class_=(date, str), doc="""
+      Dates to make unavailable for selection.""")
 
-    end = param.CalendarDate(default=None)
+    enabled_dates = param.List(default=None, class_=(date, str), doc="""
+      Dates to make available for selection.""")
 
-    disabled_dates = param.List(default=None, class_=(date, str))
+    enable_time = param.Boolean(default=True, doc="""
+      Enable editing of the time in the widget.""")
 
-    enabled_dates = param.List(default=None, class_=(date, str))
+    enable_seconds = param.Boolean(default=True, doc="""
+      Enable editing of the seconds in the widget.""")
 
-    enable_time = param.Boolean(default=True)
+    end = param.Date(default=None, doc="""
+      Inclusive upper bound of the allowed date selection. Note that while
+      the parameter accepts datetimes the time portion of the range
+      is ignored.""")
 
-    enable_seconds = param.Boolean(default=True)
+    military_time = param.Boolean(default=True, doc="""
+      Whether to display time in 24 hour format.""")
 
-    military_time = param.Boolean(default=True)
+    start = param.Date(default=None, doc="""
+      Inclusive lower bound of the allowed date selection. Note that while
+      the parameter accepts datetimes the time portion of the range
+      is ignored.""")
 
     _source_transforms = {'value': None, 'start': None, 'end': None, 'mode': None}
 
@@ -218,13 +329,28 @@ class _DatetimePickerBase(Widget):
 
 
 class DatetimePicker(_DatetimePickerBase):
+    """
+    The `DatetimePicker` allows selecting selecting a `datetime` value using a
+    textbox and a datetime-picking utility.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/DatetimePicker.html
+
+    :Example:
+
+    >>> DatetimePicker(
+    ...    value=datetime(2025,1,1,22,0),
+    ...    start=date(2025,1,1), end=date(2025,12,31),
+    ...    military_time=True, name='Date and time'
+    ... )
+    """
+
 
     value = param.Date(default=None)
 
     mode = param.String('single', constant=True)
 
     def _serialize_value(self, value):
-        if isinstance(value, string_types) and value:
+        if isinstance(value, str) and value:
             value = datetime.strptime(value, r'%Y-%m-%d %H:%M:%S')
 
             # Hour, minute and seconds can be increased after end is reached.
@@ -243,12 +369,28 @@ class DatetimePicker(_DatetimePickerBase):
 
 
 class DatetimeRangePicker(_DatetimePickerBase):
-    value = param.DateRange(default=None)
+    """
+    The `DatetimeRangePicker` allows selecting selecting a `datetime` range
+    using a text box and a datetime-range-picking utility.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/DatetimeRangePicker.html
+
+    :Example:
+
+    >>> DatetimeRangePicker(
+    ...    value=(datetime(2025,1,1,22,0), datetime(2025,1,2,22,0)),
+    ...    start=date(2025,1,1), end=date(2025,12,31),
+    ...    military_time=True, name='Datetime Range'
+    ... )
+    """
+
+    value = param.DateRange(default=None, doc="""
+        The current value""")
 
     mode = param.String('range', constant=True)
 
     def _serialize_value(self, value):
-        if isinstance(value, string_types) and value:
+        if isinstance(value, str) and value:
             value = [
                 datetime.strptime(value, r'%Y-%m-%d %H:%M:%S')
                 for value in value.split(' to ')
@@ -276,6 +418,16 @@ class DatetimeRangePicker(_DatetimePickerBase):
 
 
 class ColorPicker(Widget):
+    """
+    The `ColorPicker` widget allows selecting a hexidecimal RGB color value
+    using the browser’s color-picking widget.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/ColorPicker.html
+
+    :Example:
+
+    >>> ColorPicker(name='Color', value='#99ef78')
+    """
 
     value = param.Color(default=None, doc="""
         The selected color""")
@@ -288,12 +440,12 @@ class ColorPicker(Widget):
 class _NumericInputBase(Widget):
 
     value = param.Number(default=0, allow_None=True, doc="""
-        The initial value of the spinner.""")
+        The current value of the spinner.""")
 
     placeholder = param.String(default='0', doc="""
         Placeholder for empty input field.""")
 
-    format = param.ClassSelector(default=None, class_=string_types+(TickFormatter,), doc="""
+    format = param.ClassSelector(default=None, class_=(str, TickFormatter,), doc="""
         Allows defining a custom format string or bokeh TickFormatter.""")
 
     start = param.Parameter(default=None, allow_None=True, doc="""
@@ -312,7 +464,7 @@ class _NumericInputBase(Widget):
 class _IntInputBase(_NumericInputBase):
 
     value = param.Integer(default=0, allow_None=True, doc="""
-        The initial value of the spinner.""")
+        The current value of the spinner.""")
 
     start = param.Integer(default=None, allow_None=True, doc="""
         Optional minimum allowable value.""")
@@ -329,7 +481,7 @@ class _IntInputBase(_NumericInputBase):
 class _FloatInputBase(_NumericInputBase):
 
     value = param.Number(default=0, allow_None=True, doc="""
-        The initial value of the spinner.""")
+        The current value of the spinner.""")
 
     start = param.Number(default=None, allow_None=True, doc="""
         Optional minimum allowable value.""")
@@ -376,6 +528,13 @@ class _SpinnerBase(_NumericInputBase):
 
         return super()._update_model(events, msg, root, model, doc, comm)
 
+    def _process_param_change(self, msg):
+        # Workaround for -inf serialization errors
+        if 'value' in msg and msg['value'] == float('-inf'):
+            msg['value'] = None
+            msg['value_throttled'] = None
+        return super()._process_param_change(msg)
+
     def _process_property_change(self, msg):
         if config.throttled:
             if "value" in msg:
@@ -386,17 +545,48 @@ class _SpinnerBase(_NumericInputBase):
 
 
 class IntInput(_SpinnerBase, _IntInputBase):
+    """
+    The `IntInput` allows selecting an integer value using a spinbox.
+    
+    It behaves like a slider except that lower and upper bounds are optional
+    and a specific value can be entered. The value can be changed using the
+    keyboard (up, down, page up, page down), mouse wheel and arrow buttons.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/IntInput.html
 
-    step = param.Integer(default=1)
+    :Example:
 
-    value_throttled = param.Integer(default=None, constant=True)
+    >>> IntInput(name='Value', value=100, start=0, end=1000, step=10)
+    """
+
+    step = param.Integer(default=1, doc="""
+        The step size.""")
+
+    value_throttled = param.Integer(default=None, constant=True, doc="""
+        The current value. Updates only on `<enter>` or when the widget looses focus.""")
 
 
 class FloatInput(_SpinnerBase, _FloatInputBase):
+    """
+    The `FloatInput` allows selecting a floating point value using a spinbox.
+    
+    It behaves like a slider except that the lower and upper bounds are
+    optional and a specific value can be entered. The value can be changed
+    using the keyboard (up, down, page up, page down), mouse wheel and arrow
+    buttons.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/FloatInput.html
 
-    step = param.Number(default=0.1)
+    :Example:
 
-    value_throttled = param.Number(default=None, constant=True)
+    >>> FloatInput(name='Value', value=5., step=1e-1, start=0, end=10)
+    """
+
+    step = param.Number(default=0.1, doc="""
+        The step size.""")
+
+    value_throttled = param.Number(default=None, constant=True, doc="""
+        The current value. Updates only on `<enter>` or when the widget looses focus.""")
 
 
 class NumberInput(_SpinnerBase):
@@ -415,8 +605,20 @@ Spinner = NumberInput
 
 class LiteralInput(Widget):
     """
-    LiteralInput allows declaring Python literals using a text
-    input widget. Optionally a type may be declared.
+    The `LiteralInput` allows declaring Python literals using a text
+    input widget. 
+    
+    A *literal* is some specific primitive value of type `str`
+    , `int`, `float`, `bool` etc or a `dict`, `list`, `tuple`, `set` etc of
+    primitive values.
+    
+    Optionally the literal `type` may be declared.
+
+    Reference: https://panel.holoviz.org/reference/widgets/LiteralInput.html
+
+    :Example:
+
+    >>> LiteralInput(name='Dictionary', value={'key': [1, 2, 3]}, type=dict)
     """
 
     serializer = param.ObjectSelector(default='ast', objects=['ast', 'json'], doc="""
@@ -493,32 +695,110 @@ class LiteralInput(Widget):
         msg = super()._process_param_change(msg)
         if 'value' in msg:
             value = msg['value']
-            if isinstance(value, string_types):
+            if isinstance(value, str):
                 value = repr(value)
             elif self.serializer == 'json':
                 value = json.dumps(value)
             else:
-                value = '' if value is None else as_unicode(value)
+                value = '' if value is None else str(value)
             msg['value'] = value
         msg['title'] = self.name
         return msg
 
 
+class ArrayInput(LiteralInput):
+    """
+    The `ArrayInput` allows rendering and editing NumPy arrays in a text
+    input widget.
+    
+    Arrays larger than the `max_array_size` will be summarized and editing
+    will be disabled.
+
+    Reference: https://panel.holoviz.org/reference/widgets/ArrayInput.html
+
+    :Example:
+
+    >>> To be determined ...
+    """
+
+    max_array_size = param.Number(default=1000, doc="""
+        Arrays larger than this limit will be allowed in Python but
+        will not be serialized into JavaScript. Although such large
+        arrays will thus not be editable in the widget, such a
+        restriction helps avoid overwhelming the browser and lets
+        other widgets remain usable.""")
+
+    _rename = dict(LiteralInput._rename, max_array_size=None)
+
+    _source_transforms = {'value': None}
+
+    def __init__(self, **params):
+        super().__init__(**params)
+        self._auto_disabled = False
+
+    def _process_property_change(self, msg):
+        msg = super()._process_property_change(msg)
+        if 'value' in msg and isinstance(msg['value'], list):
+            msg['value'] = np.asarray(msg['value'])
+        return msg
+
+    def _process_param_change(self, msg):
+        if msg.get('disabled', False):
+            self._auto_disabled = False
+        value = msg.get('value')
+        if value is None:
+            return super()._process_param_change(msg)
+        if value.size <= self.max_array_size:
+            msg['value'] = value.tolist()
+            # If array is no longer larger than max_array_size
+            # unset disabled
+            if self.disabled and self._auto_disabled:
+                self.disabled = False
+                msg['disabled'] = False
+                self._auto_disabled = False
+        else:
+            msg['value'] = np.array2string(
+                msg['value'], separator=',',
+                threshold=self.max_array_size
+            )
+            if not self.disabled:
+                self.param.warning(
+                    f"Number of array elements ({value.size}) exceeds "
+                    f"`max_array_size` ({self.max_array_size}), editing "
+                    "will be disabled."
+                )
+                self.disabled = True
+                msg['disabled'] = True
+                self._auto_disabled = True
+        return super()._process_param_change(msg)
+
+
 class DatetimeInput(LiteralInput):
     """
-    DatetimeInput allows declaring Python literals using a text
-    input widget. Optionally a type may be declared.
+    The `DatetimeInput` allows specifying Python `datetime` like values using
+    a text input widget.
+    
+    An optional `type` may be declared.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/DatetimeInput.html
+
+    :Example:
+
+    >>> DatetimeInput(name='Datetime', value=datetime(2019, 2, 8))
     """
 
+    value = param.Date(default=None, doc="""
+        The current value""")
+
+    start = param.Date(default=None, doc="""
+        Inclusive lower bound of the allowed date selection""")
+
+    end = param.Date(default=None, doc="""
+        Inclusive upper bound of the allowed date selection""")
+
     format = param.String(default='%Y-%m-%d %H:%M:%S', doc="""
         Datetime format used for parsing and formatting the datetime.""")
 
-    value = param.Date(default=None)
-
-    start = param.Date(default=None)
-
-    end = param.Date(default=None)
-
     type = datetime
 
     _source_transforms = {'value': None, 'start': None, 'end': None}
@@ -578,12 +858,29 @@ class DatetimeInput(LiteralInput):
 
 
 class DatetimeRangeInput(CompositeWidget):
+    """
+    The `DatetimeRangeInput` widget allows selecting a `datetime` range using
+    two `DatetimeInput` widgets, which return a `tuple` range.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/DatetimeRangeInput.html
 
-    value = param.Tuple(default=(None, None), length=2)
+    :Example:
 
-    start = param.Date(default=None)
+    >>> DatetimeRangeInput(
+    ...     name='Datetime Range',
+    ...     value=(datetime(2017, 1, 1), datetime(2018, 1, 10)),
+    ...     start=datetime(2017, 1, 1), end=datetime(2019, 1, 1),
+    ... )
+    """
 
-    end = param.Date(default=None)
+    value = param.Tuple(default=(None, None), length=2, doc="""
+        The current value""")
+
+    start = param.Date(default=None, doc="""
+        Inclusive lower bound of the allowed date selection""")
+
+    end = param.Date(default=None, doc="""
+        Inclusive upper bound of the allowed date selection""")
 
     format = param.String(default='%Y-%m-%d %H:%M:%S', doc="""
         Datetime format used for parsing and formatting the datetime.""")
@@ -635,18 +932,31 @@ class DatetimeRangeInput(CompositeWidget):
             return
         try:
             self._updating = True
-            params = {k: v for k, v in self.param.get_param_values() if k in filters}
+            params = {k: v for k, v in self.param.values().items() if k in filters}
             start_params = dict(params, value=self.value[0])
             end_params = dict(params, value=self.value[1])
-            self._start.param.set_param(**start_params)
-            self._end.param.set_param(**end_params)
+            self._start.param.update(**start_params)
+            self._end.param.update(**end_params)
         finally:
             self._updating = False
 
 
 class Checkbox(Widget):
+    """
+    The `Checkbox` allows toggling a single condition between `True`/`False`
+    states by ticking a checkbox.
+    
+    This widget is interchangeable with the `Toggle` widget.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/Checkbox.html
 
-    value = param.Boolean(default=False)
+    :Example:
+
+    >>> Checkbox(name='Works with the tools you know and love', value=True)
+    """
+
+    value = param.Boolean(default=False, doc="""
+        The current value""")
 
     _supports_embed = True
 
