@@ -5,6 +5,7 @@ from a list of options.
 import re
 
 from collections import OrderedDict
+import itertools
 
 import param
 
@@ -12,12 +13,12 @@ from bokeh.models.widgets import (
     AutocompleteInput as _BkAutocompleteInput, CheckboxGroup as _BkCheckboxGroup,
     CheckboxButtonGroup as _BkCheckboxButtonGroup, MultiSelect as _BkMultiSelect,
     RadioButtonGroup as _BkRadioButtonGroup, RadioGroup as _BkRadioBoxGroup,
-    Select as _BkSelect, MultiChoice as _BkMultiChoice
+    MultiChoice as _BkMultiChoice
 )
 
 from ..layout import Column, VSpacer
-from ..models import SingleSelect as _BkSingleSelect
-from ..util import as_unicode, isIn, indexOf
+from ..models import SingleSelect as _BkSingleSelect, CustomSelect
+from ..util import isIn, indexOf
 from .base import Widget, CompositeWidget
 from .button import _ButtonBase, Button
 from .input import TextInput, TextAreaInput
@@ -31,7 +32,7 @@ class SelectBase(Widget):
 
     @property
     def labels(self):
-        return [as_unicode(o) for o in self.options]
+        return [str(o) for o in self.options]
 
     @property
     def values(self):
@@ -94,7 +95,7 @@ class SingleSelectBase(SelectBase):
 
     @property
     def unicode_values(self):
-        return [as_unicode(v) for v in self.values]
+        return [str(v) for v in self.values]
 
     def _process_property_change(self, msg):
         msg = super()._process_property_change(msg)
@@ -124,29 +125,167 @@ class SingleSelectBase(SelectBase):
 
 
 class Select(SingleSelectBase):
+    """
+    The `Select` widget allows selecting a value from a list or dictionary of
+    `options` by selecting it from a dropdown menu or selection area. 
+    
+    It falls into the broad category of single-value, option-selection widgets
+    that provide a compatible API and include the `RadioBoxGroup`,
+    `AutocompleteInput` and `DiscreteSlider` widgets.
+
+    Reference: https://panel.holoviz.org/reference/widgets/Select.html
+
+    :Example:
+
+    >>> Select(name='Study', options=['Biology', 'Chemistry', 'Physics'])
+    """
+
+    disabled_options = param.List(default=[], doc="""
+        Optional list of ``options`` that are disabled, i.e. unusable and
+        un-clickable. If ``options`` is a dictionary the list items must be
+        dictionary values.""")
+
+    groups = param.Dict(default=None, doc="""
+        Dictionary whose keys are used to visually group the options
+        and whose values are either a list or a dictionary of options
+        to select from. Mutually exclusive with ``options``  and valid only
+        if ``size`` is 1.""")
 
     size = param.Integer(default=1, bounds=(1, None), doc="""
         Declares how many options are displayed at the same time.
         If set to 1 displays options as dropdown otherwise displays
         scrollable area.""")
 
-    _source_transforms = {'size': None}
+    _source_transforms = {'size': None, 'groups': None}
 
     @property
     def _widget_type(self):
-        return _BkSelect if self.size == 1 else _BkSingleSelect
+        return CustomSelect if self.size == 1 else _BkSingleSelect
 
     def __init__(self, **params):
         super().__init__(**params)
         if self.size == 1:
             self.param.size.constant = True
+        self._callbacks.extend([
+            self.param.watch(
+                self._validate_options_groups,
+                ['options', 'groups']
+            ),
+            self.param.watch(
+                self._validate_disabled_options,
+                ['options', 'disabled_options', 'value']
+            ),
+        ])
+        self._validate_options_groups()
+        self._validate_disabled_options()
+
+    def _validate_disabled_options(self, *events):
+        if self.disabled_options and self.disabled_options == self.values:
+            raise ValueError(
+                f'All the options of a {type(self).__name__} '
+                'widget cannot be disabled.'
+            )
+        not_in_opts = [
+            dopts
+            for dopts in self.disabled_options
+            if dopts not in (self.values or [])
+        ]
+        if not_in_opts:
+            raise ValueError(
+                f'Cannot disable non existing options of {type(self).__name__}: {not_in_opts}'
+            )
+        if len(events) == 1:
+            if events[0].name == 'value' and self.value in self.disabled_options:
+                raise ValueError(
+                    f'Cannot set the value of {type(self).__name__} to '
+                    f'{self.value!r} as it is a disabled option.'
+                )
+            elif events[0].name == 'disabled_options' and self.value in self.disabled_options:
+                raise ValueError(
+                    f'Cannot set disabled_options of {type(self).__name__} to a list that '
+                    f'includes the current value {self.value!r}.'
+                )
+        if self.value in self.disabled_options:
+            raise ValueError(
+                f'Cannot initialize {type(self).__name__} with value {self.value!r} '
+                'as it is one of the disabled options.'
+            )
+
+
+    def _validate_options_groups(self, *events):
+        if self.options and self.groups:
+            raise ValueError(
+                f'{type(self).__name__} options and groups parameters '
+                'are mutually exclusive.'
+            )
+        if self.size > 1 and self.groups:
+            raise ValueError(
+                f'{type(self).__name__} with size > 1 doe not support the'
+                ' `groups` parameter, use `options` instead.'
+            )
 
     def _process_param_change(self, msg):
         msg = super()._process_param_change(msg)
         if msg.get('size') == 1:
             msg.pop('size')
+        groups = msg.pop('groups', None)
+        if groups is not None:
+            if (all(isinstance(values, dict) for values in groups.values()) is False
+               and  all(isinstance(values, list) for values in groups.values()) is False):
+                raise ValueError(
+                    'The values of the groups dictionary must be all of '
+                    'the dictionary or the list type.'
+                )
+            labels, values = self.labels, self.values
+            unique = len(set(self.unicode_values)) == len(labels)
+            if groups:
+                if isinstance(next(iter(self.groups.values())), dict):
+                    if unique:
+                        options = {
+                            group: [(str(value), label) for label, value in subd.items()]
+                            for group, subd in groups.items()
+                        }
+                    else:
+                        options = {
+                            group: [str(v) for v in self.groups[group]]
+                            for group in groups.keys()
+                        }
+                    msg['options'] = options
+                else:
+                    msg['options'] = {
+                        group: [(str(value), str(value)) for value in values]
+                        for group, values in groups.items()
+                    }
+            val = self.value
+            if values:
+                if not isIn(val, values):
+                    self.value = values[0]
+            else:
+                self.value = None
         return msg
 
+    @property
+    def labels(self):
+        if self.options:
+            return super().labels
+        else:
+            if not self.groups:
+                return {}
+            else:
+                return list(map(str, itertools.chain(*self.groups.values())))
+
+    @property
+    def values(self):
+        if self.options:
+            return super().values
+        else:
+            if not self.groups:
+                return []
+            if isinstance(next(iter(self.groups.values())), dict):
+                return [v for subd in self.groups.values() for v in subd.values()]
+            else:
+                return list(itertools.chain(*self.groups.values()))
+
 
 class _MultiSelectBase(SingleSelectBase):
 
@@ -178,6 +317,23 @@ class _MultiSelectBase(SingleSelectBase):
 
 
 class MultiSelect(_MultiSelectBase):
+    """
+    The `MultiSelect` widget allows selecting multiple values from a list of
+    `options`.
+    
+    It falls into the broad category of multi-value, option-selection widgets
+    that provide a compatible API and include the`CrossSelector`,
+    `CheckBoxGroup` and `CheckButtonGroup` widgets.
+
+    Reference: https://panel.holoviz.org/reference/widgets/MultiSelect.html
+
+    :Example:
+
+    >>> MultiSelect(
+    ...     name='Frameworks', value=['Bokeh', 'Panel'],
+    ...     options=['Bokeh', 'Dash', 'Panel', 'Streamlit', 'Voila'], size=8
+    ... )
+    """
 
     size = param.Integer(default=4, doc="""
         The number of items displayed at once (i.e. determines the
@@ -187,6 +343,27 @@ class MultiSelect(_MultiSelectBase):
 
 
 class MultiChoice(_MultiSelectBase):
+    """
+    The `MultiChoice` widget allows selecting multiple values from a list of
+    `options`. 
+    
+    It falls into the broad category of multi-value, option-selection widgets
+    that provide a compatible API and include the `MultiSelect`,
+    `CrossSelector`, `CheckBoxGroup` and `CheckButtonGroup` widgets.
+    
+    The `MultiChoice` widget provides a much more compact UI than
+    `MultiSelect`.
+
+    Reference: https://panel.holoviz.org/reference/widgets/MultiChoice.html
+
+    :Example:
+
+    >>> MultiChoice(
+    ...     name='Favourites', value=['Panel', 'hvPlot'],
+    ...     options=['Panel', 'hvPlot', 'HoloViews', 'GeoViews', 'Datashader', 'Param', 'Colorcet'],
+    ...     max_items=2
+    ... )
+    """
 
     delete_button = param.Boolean(default=True, doc="""
         Whether to display a button to delete a selected option.""")
@@ -196,7 +373,10 @@ class MultiChoice(_MultiSelectBase):
 
     option_limit = param.Integer(default=None, bounds=(1, None), doc="""
         Maximum number of options to display at once.""")
-
+    
+    search_option_limit = param.Integer(default=None, bounds=(1, None), doc="""
+        Maximum number of options to display at once if search string is entered.""")
+    
     placeholder = param.String(default='', doc="""
         String displayed when no selection has been made.""")
 
@@ -210,6 +390,26 @@ _AutocompleteInput_rename = {'name': 'title', 'options': 'completions'}
 
 
 class AutocompleteInput(Widget):
+    """
+    The `MultiChoice` widget allows selecting multiple values from a list of
+    `options`. 
+    
+    It falls into the broad category of multi-value, option-selection widgets
+    that provide a compatible API and include the `MultiSelect`,
+    `CrossSelector`, `CheckBoxGroup` and `CheckButtonGroup` widgets.
+    
+    The `MultiChoice` widget provides a much more compact UI than
+    `MultiSelect`.
+
+    Reference: https://panel.holoviz.org/reference/widgets/AutocompleteInput.html
+
+    :Example:
+
+    >>> AutocompleteInput(
+    ...     name='Study', options=['Biology', 'Chemistry', 'Physics'],
+    ...     placeholder='Write your study here ...'
+    ... )
+    """
 
     case_sensitive = param.Boolean(default=True, doc="""
         Enable or disable case sensitivity.""")
@@ -239,6 +439,13 @@ class AutocompleteInput(Widget):
 
     _rename = _AutocompleteInput_rename
 
+    def _process_param_change(self, msg):
+        msg = super()._process_param_change(msg)
+        if 'completions' in msg:
+            if self.restrict and not isIn(self.value, msg['completions']):
+                msg['value'] = self.value = ''
+        return msg
+
 
 class _RadioGroupBase(SingleSelectBase):
 
@@ -294,6 +501,27 @@ class _RadioGroupBase(SingleSelectBase):
 
 
 class RadioButtonGroup(_RadioGroupBase, _ButtonBase):
+    """
+    The `RadioButtonGroup` widget allows selecting from a list or dictionary
+    of values using a set of toggle buttons. 
+    
+    It falls into the broad category of single-value, option-selection widgets
+    that provide a compatible API and include the `RadioBoxGroup`, `Select`,
+    and `DiscreteSlider` widgets.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/RadioButtonGroup.html
+
+    :Example:
+
+    >>> RadioButtonGroup(
+    ...     name='Plotting library', options=['Matplotlib', 'Bokeh', 'Plotly'],
+    ...     button_type='success'
+    ... )
+    """
+
+    orientation = param.Selector(default='horizontal',
+        objects=['horizontal', 'vertical'], doc="""
+        Button group orientation, either 'horizontal' (default) or 'vertical'.""")
 
     _widget_type = _BkRadioButtonGroup
 
@@ -302,6 +530,22 @@ class RadioButtonGroup(_RadioGroupBase, _ButtonBase):
 
 
 class RadioBoxGroup(_RadioGroupBase):
+    """
+    The `RadioBoxGroup` widget allows selecting from a list or dictionary of
+    values using a set of checkboxes.
+    
+    It falls into the broad category of single-value, option-selection widgets
+    that provide a compatible API and include the `RadioButtonGroup`, `Select`
+    and `DiscreteSlider` widgets.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/RadioBoxGroup.html
+
+    :Example:
+
+    >>> RadioBoxGroup(
+    ...     name='Sponsor', options=['Anaconda', 'Blackstone'], inline=True
+    ... )
+    """
 
     inline = param.Boolean(default=False, doc="""
         Whether the items be arrange vertically (``False``) or
@@ -328,7 +572,7 @@ class _CheckGroupBase(SingleSelectBase):
     __abstract = True
 
     def _process_param_change(self, msg):
-        msg = super(SingleSelectBase, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         values = self.values
         if 'active' in msg:
             msg['active'] = [indexOf(v, values) for v in msg['active']
@@ -337,6 +581,8 @@ class _CheckGroupBase(SingleSelectBase):
             msg['labels'] = self.labels
             if any(not isIn(v, values) for v in self.value):
                 self.value = [v for v in self.value if isIn(v, values)]
+            msg["active"] = [indexOf(v, values) for v in self.value
+                             if isIn(v, values)]
         msg.pop('title', None)
         return msg
 
@@ -350,11 +596,49 @@ class _CheckGroupBase(SingleSelectBase):
 
 
 class CheckButtonGroup(_CheckGroupBase, _ButtonBase):
+    """
+    The `CheckButtonGroup` widget allows selecting between a list of options
+    by toggling the corresponding buttons.
+    
+    It falls into the broad category of multi-option selection widgets that
+    provide a compatible API and include the `MultiSelect`, `CrossSelector`
+    and `CheckBoxGroup` widgets.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/CheckButtonGroup.html
+
+    :Example:
+
+    >>> CheckButtonGroup(
+    ...     name='Regression Models', value=['Lasso', 'Ridge'],
+    ...     options=['Lasso', 'Linear', 'Ridge', 'Polynomial']
+    ... )
+    """
+
+    orientation = param.Selector(default='horizontal',
+        objects=['horizontal', 'vertical'], doc="""
+        Button group orientation, either 'horizontal' (default) or 'vertical'.""")
 
     _widget_type = _BkCheckboxButtonGroup
 
 
 class CheckBoxGroup(_CheckGroupBase):
+    """
+    The `CheckBoxGroup` widget allows selecting between a list of options by
+    ticking the corresponding checkboxes.
+    
+    It falls into the broad category of multi-option selection widgets that
+    provide a compatible API and include the `MultiSelect`, `CrossSelector`
+    and `CheckButtonGroup` widgets.
+    
+    Reference: https://panel.holoviz.org/reference/widgets/CheckBoxGroup.html
+
+    :Example:
+
+    >>> CheckBoxGroup(
+    ...     name='Fruits', value=['Apple', 'Pear'], options=['Apple', 'Banana', 'Pear', 'Strawberry'],
+    ...     inline=True
+    ... )
+    """
 
     inline = param.Boolean(default=False, doc="""
         Whether the items be arrange vertically (``False``) or
@@ -415,6 +699,15 @@ class CrossSelector(CompositeWidget, MultiSelect):
     A composite widget which allows selecting from a list of items
     by moving them between two lists. Supports filtering values by
     name to select them in bulk.
+
+    Reference: https://panel.holoviz.org/reference/widgets/CrossSelector.html
+
+    :Example:
+
+    >>> CrossSelector(
+    ...     name='Fruits', value=['Apple', 'Pear'], 
+    ...     options=['Apple', 'Banana', 'Pear', 'Strawberry']
+    ... )
     """
 
     width = param.Integer(default=600, allow_None=True, doc="""
@@ -491,6 +784,8 @@ class CrossSelector(CompositeWidget, MultiSelect):
         self._selections = {False: [], True: []}
         self._query = {False: '', True: ''}
 
+        self._update_disabled()
+
     @param.depends('size', watch=True)
     def _update_size(self):
         self._lists[False].size = self.size
