@@ -277,6 +277,12 @@ class SpeechToText(Widget):
     It wraps the HTML5 SpeechRecognition API.  See
     https://developer.mozilla.org/en-US/docs/Web/API/SpeechRecognition
 
+    Reference: https://panel.holoviz.org/reference/widgets/SpeechToText.html
+
+    :Example:
+
+    >>> SpeechToText(button_type="light")
+    
     This functionality is **experimental** and only supported by
     Chrome and a few other browsers.  Checkout
     https://caniuse.com/speech-recognition for a up to date list of
